#!/usr/bin/env python
#
# Copyright 2010 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""Tests for protorpc.service_handlers."""

__author__ = 'rafek@google.com (Rafe Kaplan)'


import cgi
import cStringIO
import os
import re
import sys
import unittest
import urllib

from protorpc import messages
from protorpc import protobuf
from protorpc import protojson
from protorpc import protourlencode
from protorpc import message_types
from protorpc import registry
from protorpc import remote
from protorpc import test_util
from protorpc import util
from protorpc import webapp_test_util
from protorpc.webapp import forms
from protorpc.webapp import service_handlers
from protorpc.webapp.google_imports import webapp

import mox

package = 'test_package'


class ModuleInterfaceTest(test_util.ModuleInterfaceTest,
                          test_util.TestCase):

  MODULE = service_handlers


class Enum1(messages.Enum):
  """A test enum class."""

  VAL1 = 1
  VAL2 = 2
  VAL3 = 3


class Request1(messages.Message):
  """A test request message type."""

  integer_field = messages.IntegerField(1)
  string_field = messages.StringField(2)
  enum_field = messages.EnumField(Enum1, 3)


class Response1(messages.Message):
  """A test response message type."""

  integer_field = messages.IntegerField(1)
  string_field = messages.StringField(2)
  enum_field = messages.EnumField(Enum1, 3)


class SuperMessage(messages.Message):
  """A test message with a nested message field."""

  sub_message = messages.MessageField(Request1, 1)
  sub_messages = messages.MessageField(Request1, 2, repeated=True)


class SuperSuperMessage(messages.Message):
  """A test message with two levels of nested."""

  sub_message = messages.MessageField(SuperMessage, 1)
  sub_messages = messages.MessageField(Request1, 2, repeated=True)


class RepeatedMessage(messages.Message):
  """A test message with a repeated field."""

  ints = messages.IntegerField(1, repeated=True)
  strings = messages.StringField(2, repeated=True)
  enums = messages.EnumField(Enum1, 3, repeated=True)


class Service(object):
  """A simple service that takes a Request1 and returns Request2."""

  @remote.method(Request1, Response1)
  def method1(self, request):
    response = Response1()
    if hasattr(request, 'integer_field'):
      response.integer_field = request.integer_field
    if hasattr(request, 'string_field'):
      response.string_field = request.string_field
    if hasattr(request, 'enum_field'):
      response.enum_field = request.enum_field
    return response

  @remote.method(RepeatedMessage, RepeatedMessage)
  def repeated_method(self, request):
    response = RepeatedMessage()
    if hasattr(request, 'ints'):
      response = request.ints
    return response

  def not_remote(self):
    pass


def VerifyResponse(test,
                   response,
                   expected_status,
                   expected_status_message,
                   expected_content,
                   expected_content_type='application/x-www-form-urlencoded'):
  def write(content):
    if expected_content == '':
      test.assertEquals(util.pad_string(''), content)
    else:
      test.assertNotEquals(-1, content.find(expected_content),
                           'Expected to find:\n%s\n\nActual content: \n%s' % (
                             expected_content, content))

  def start_response(response, headers):
    status, message = response.split(' ', 1)
    test.assertEquals(expected_status, status)
    test.assertEquals(expected_status_message, message)
    for name, value in headers:
      if name.lower() == 'content-type':
        test.assertEquals(expected_content_type, value)
    for name, value in headers:
      if name.lower() == 'x-content-type-options':
        test.assertEquals('nosniff', value)
      elif name.lower() == 'content-type':
        test.assertFalse(value.lower().startswith('text/html'))
    return write

  response.wsgi_write(start_response)


class ServiceHandlerFactoryTest(test_util.TestCase):
  """Tests for the service handler factory."""

  def testAllRequestMappers(self):
    """Test all_request_mappers method."""
    configuration = service_handlers.ServiceHandlerFactory(Service)
    mapper1 = service_handlers.RPCMapper(['whatever'], 'whatever', None)
    mapper2 = service_handlers.RPCMapper(['whatever'], 'whatever', None)

    configuration.add_request_mapper(mapper1)
    self.assertEquals([mapper1], list(configuration.all_request_mappers()))

    configuration.add_request_mapper(mapper2)
    self.assertEquals([mapper1, mapper2],
                      list(configuration.all_request_mappers()))

  def testServiceFactory(self):
    """Test that service_factory attribute is set."""
    handler_factory = service_handlers.ServiceHandlerFactory(Service)
    self.assertEquals(Service, handler_factory.service_factory)

  def testFactoryMethod(self):
    """Test that factory creates correct instance of class."""
    factory = service_handlers.ServiceHandlerFactory(Service)
    handler = factory()

    self.assertTrue(isinstance(handler, service_handlers.ServiceHandler))
    self.assertTrue(isinstance(handler.service, Service))

  def testMapping(self):
    """Test the mapping method."""
    factory = service_handlers.ServiceHandlerFactory(Service)
    path, mapped_factory = factory.mapping('/my_service')

    self.assertEquals(r'(/my_service)' + service_handlers._METHOD_PATTERN, path)
    self.assertEquals(id(factory), id(mapped_factory))
    match = re.match(path, '/my_service.my_method')
    self.assertEquals('/my_service', match.group(1))
    self.assertEquals('my_method', match.group(2))

    path, mapped_factory = factory.mapping('/my_service/nested')
    self.assertEquals('(/my_service/nested)' +
                      service_handlers._METHOD_PATTERN, path)
    match = re.match(path, '/my_service/nested.my_method')
    self.assertEquals('/my_service/nested', match.group(1))
    self.assertEquals('my_method', match.group(2))

  def testRegexMapping(self):
    """Test the mapping method using a regex."""
    factory = service_handlers.ServiceHandlerFactory(Service)
    path, mapped_factory = factory.mapping('.*/my_service')

    self.assertEquals(r'(.*/my_service)' + service_handlers._METHOD_PATTERN, path)
    self.assertEquals(id(factory), id(mapped_factory))
    match = re.match(path, '/whatever_preceeds/my_service.my_method')
    self.assertEquals('/whatever_preceeds/my_service', match.group(1))
    self.assertEquals('my_method', match.group(2))
    match = re.match(path, '/something_else/my_service.my_other_method')
    self.assertEquals('/something_else/my_service', match.group(1))
    self.assertEquals('my_other_method', match.group(2))

  def testMapping_BadPath(self):
    """Test bad parameterse to the mapping method."""
    factory = service_handlers.ServiceHandlerFactory(Service)
    self.assertRaises(ValueError, factory.mapping, '/my_service/')

  def testDefault(self):
    """Test the default factory convenience method."""
    handler_factory = service_handlers.ServiceHandlerFactory.default(
        Service,
        parameter_prefix='my_prefix.')

    self.assertEquals(Service, handler_factory.service_factory)

    mappers = handler_factory.all_request_mappers()

    # Verify Protobuf encoded mapper.
    protobuf_mapper = next(mappers)
    self.assertTrue(isinstance(protobuf_mapper,
                               service_handlers.ProtobufRPCMapper))

    # Verify JSON encoded mapper.
    json_mapper = next(mappers)
    self.assertTrue(isinstance(json_mapper,
                               service_handlers.JSONRPCMapper))

    # Should have no more mappers.
    self.assertRaises(StopIteration, mappers.next)


class ServiceHandlerTest(webapp_test_util.RequestHandlerTestBase):
  """Test the ServiceHandler class."""

  def setUp(self):
    self.mox = mox.Mox()
    self.service_factory = Service
    self.remote_host = 'remote.host.com'
    self.server_host = 'server.host.com'
    self.ResetRequestHandler()

    self.request = Request1()
    self.request.integer_field = 1
    self.request.string_field = 'a'
    self.request.enum_field = Enum1.VAL1

  def ResetRequestHandler(self):
    super(ServiceHandlerTest, self).setUp()

  def CreateService(self):
    return self.service_factory()

  def CreateRequestHandler(self):
    self.rpc_mapper1 = self.mox.CreateMock(service_handlers.RPCMapper)
    self.rpc_mapper1.http_methods = set(['POST'])
    self.rpc_mapper1.content_types = set(['application/x-www-form-urlencoded'])
    self.rpc_mapper1.default_content_type = 'application/x-www-form-urlencoded'
    self.rpc_mapper2 = self.mox.CreateMock(service_handlers.RPCMapper)
    self.rpc_mapper2.http_methods = set(['GET'])
    self.rpc_mapper2.content_types = set(['application/json'])
    self.rpc_mapper2.default_content_type = 'application/json'
    self.factory = service_handlers.ServiceHandlerFactory(
        self.CreateService)
    self.factory.add_request_mapper(self.rpc_mapper1)
    self.factory.add_request_mapper(self.rpc_mapper2)
    return self.factory()

  def GetEnvironment(self):
    """Create handler to test."""
    environ = super(ServiceHandlerTest, self).GetEnvironment()
    if self.remote_host:
      environ['REMOTE_HOST'] = self.remote_host
    if self.server_host:
      environ['SERVER_HOST'] = self.server_host
    return environ

  def VerifyResponse(self, *args, **kwargs):
    VerifyResponse(self,
                   self.response,
                   *args, **kwargs)

  def ExpectRpcError(self, mapper, state, error_message, error_name=None):
    mapper.build_response(self.handler,
                          remote.RpcStatus(state=state,
                                           error_message=error_message,
                                           error_name=error_name))

  def testRedirect(self):
    """Test that redirection is disabled."""
    self.assertRaises(NotImplementedError, self.handler.redirect, '/')

  def testFirstMapper(self):
    """Make sure service attribute works when matches first RPCMapper."""
    self.rpc_mapper1.build_request(
        self.handler, Request1).AndReturn(self.request)

    def build_response(handler, response):
      output = '%s %s %s' % (response.integer_field,
                             response.string_field,
                             response.enum_field)
      handler.response.headers['content-type'] = (
        'application/x-www-form-urlencoded')
      handler.response.out.write(output)
    self.rpc_mapper1.build_response(
        self.handler, mox.IsA(Response1)).WithSideEffects(build_response)

    self.mox.ReplayAll()

    self.handler.handle('POST', '/my_service', 'method1')

    self.VerifyResponse('200', 'OK', '1 a VAL1')

    self.mox.VerifyAll()

  def testSecondMapper(self):
    """Make sure service attribute works when matches first RPCMapper.

    Demonstrates the multiplicity of the RPCMapper configuration.
    """
    self.rpc_mapper2.build_request(
        self.handler, Request1).AndReturn(self.request)

    def build_response(handler, response):
      output = '%s %s %s' % (response.integer_field,
                             response.string_field,
                             response.enum_field)
      handler.response.headers['content-type'] = (
        'application/x-www-form-urlencoded')
      handler.response.out.write(output)
    self.rpc_mapper2.build_response(
        self.handler, mox.IsA(Response1)).WithSideEffects(build_response)

    self.mox.ReplayAll()

    self.handler.request.headers['Content-Type'] = 'application/json'
    self.handler.handle('GET', '/my_service', 'method1')

    self.VerifyResponse('200', 'OK', '1 a VAL1')

    self.mox.VerifyAll()

  def testCaseInsensitiveContentType(self):
    """Ensure that matching content-type is case insensitive."""
    request = Request1()
    request.integer_field = 1
    request.string_field = 'a'
    request.enum_field = Enum1.VAL1
    self.rpc_mapper1.build_request(self.handler,
                                   Request1).AndReturn(self.request)

    def build_response(handler, response):
      output = '%s %s %s' % (response.integer_field,
                             response.string_field,
                             response.enum_field)
      handler.response.out.write(output)
      handler.response.headers['content-type'] = 'text/plain'
    self.rpc_mapper1.build_response(
        self.handler, mox.IsA(Response1)).WithSideEffects(build_response)

    self.mox.ReplayAll()

    self.handler.request.headers['Content-Type'] = ('ApPlIcAtIoN/'
                                                    'X-wWw-FoRm-UrLeNcOdEd')

    self.handler.handle('POST', '/my_service', 'method1')

    self.VerifyResponse('200', 'OK', '1 a VAL1', 'text/plain')

    self.mox.VerifyAll()

  def testContentTypeWithParameters(self):
    """Test that content types have parameters parsed out."""
    request = Request1()
    request.integer_field = 1
    request.string_field = 'a'
    request.enum_field = Enum1.VAL1
    self.rpc_mapper1.build_request(self.handler,
                                   Request1).AndReturn(self.request)

    def build_response(handler, response):
      output = '%s %s %s' % (response.integer_field,
                             response.string_field,
                             response.enum_field)
      handler.response.headers['content-type'] = (
        'application/x-www-form-urlencoded')
      handler.response.out.write(output)
    self.rpc_mapper1.build_response(
        self.handler, mox.IsA(Response1)).WithSideEffects(build_response)

    self.mox.ReplayAll()

    self.handler.request.headers['Content-Type'] = ('application/'
                                                    'x-www-form-urlencoded' +
                                                    '; a=b; c=d')

    self.handler.handle('POST', '/my_service', 'method1')

    self.VerifyResponse('200', 'OK', '1 a VAL1')

    self.mox.VerifyAll()

  def testContentFromHeaderOnly(self):
    """Test getting content-type from HTTP_CONTENT_TYPE directly.

    Some bad web server implementations might decide not to set CONTENT_TYPE for
    POST requests where there is an empty body.  In these cases, need to get
    content-type directly from webob environ key HTTP_CONTENT_TYPE.
    """
    request = Request1()
    request.integer_field = 1
    request.string_field = 'a'
    request.enum_field = Enum1.VAL1
    self.rpc_mapper1.build_request(self.handler,
                                   Request1).AndReturn(self.request)

    def build_response(handler, response):
      output = '%s %s %s' % (response.integer_field,
                             response.string_field,
                             response.enum_field)
      handler.response.headers['Content-Type'] = (
        'application/x-www-form-urlencoded')
      handler.response.out.write(output)
    self.rpc_mapper1.build_response(
        self.handler, mox.IsA(Response1)).WithSideEffects(build_response)

    self.mox.ReplayAll()

    self.handler.request.headers['Content-Type'] = None
    self.handler.request.environ['HTTP_CONTENT_TYPE'] = (
      'application/x-www-form-urlencoded')

    self.handler.handle('POST', '/my_service', 'method1')

    self.VerifyResponse('200', 'OK', '1 a VAL1',
                        'application/x-www-form-urlencoded')

    self.mox.VerifyAll()

  def testRequestState(self):
    """Make sure request state is passed in to handler that supports it."""
    class ServiceWithState(object):

      initialize_request_state = self.mox.CreateMockAnything()

      @remote.method(Request1, Response1)
      def method1(self, request):
        return Response1()

    self.service_factory = ServiceWithState

    # Reset handler with new service type.
    self.ResetRequestHandler()

    self.rpc_mapper1.build_request(
        self.handler, Request1).AndReturn(Request1())

    def build_response(handler, response):
      handler.response.headers['Content-Type'] = (
        'application/x-www-form-urlencoded')
      handler.response.out.write('whatever')
    self.rpc_mapper1.build_response(
        self.handler, mox.IsA(Response1)).WithSideEffects(build_response)

    def verify_state(state):
      return (
        'remote.host.com' ==  state.remote_host and
        '127.0.0.1' == state.remote_address and
        'server.host.com' == state.server_host and
        8080 == state.server_port and
        'POST' == state.http_method and
        '/my_service' == state.service_path and
        'application/x-www-form-urlencoded' == state.headers['content-type'] and
        'dev_appserver_login="test:test@example.com:True"' ==
        state.headers['cookie'])
    ServiceWithState.initialize_request_state(mox.Func(verify_state))

    self.mox.ReplayAll()

    self.handler.handle('POST', '/my_service', 'method1')

    self.VerifyResponse('200', 'OK', 'whatever')

    self.mox.VerifyAll()

  def testRequestState_MissingHosts(self):
    """Make sure missing state environment values are handled gracefully."""
    class ServiceWithState(object):

      initialize_request_state = self.mox.CreateMockAnything()

      @remote.method(Request1, Response1)
      def method1(self, request):
        return Response1()

    self.service_factory = ServiceWithState
    self.remote_host = None
    self.server_host = None

    # Reset handler with new service type.
    self.ResetRequestHandler()

    self.rpc_mapper1.build_request(
        self.handler, Request1).AndReturn(Request1())

    def build_response(handler, response):
      handler.response.headers['Content-Type'] = (
        'application/x-www-form-urlencoded')
      handler.response.out.write('whatever')
    self.rpc_mapper1.build_response(
        self.handler, mox.IsA(Response1)).WithSideEffects(build_response)

    def verify_state(state):
      return (None is state.remote_host and
              '127.0.0.1' == state.remote_address and
              None is state.server_host and
              8080 == state.server_port)
    ServiceWithState.initialize_request_state(mox.Func(verify_state))

    self.mox.ReplayAll()

    self.handler.handle('POST', '/my_service', 'method1')

    self.VerifyResponse('200', 'OK', 'whatever')

    self.mox.VerifyAll()

  def testNoMatch_UnknownHTTPMethod(self):
    """Test what happens when no RPCMapper matches."""
    self.mox.ReplayAll()

    self.handler.handle('UNKNOWN', '/my_service', 'does_not_matter')

    self.VerifyResponse('405',
                        'Unsupported HTTP method: UNKNOWN',
                        'Method Not Allowed',
                        'text/plain; charset=utf-8')

    self.mox.VerifyAll()

  def testNoMatch_GetNotSupported(self):
    """Test what happens when GET is not supported."""
    self.mox.ReplayAll()

    self.handler.handle('GET', '/my_service', 'method1')

    self.VerifyResponse('405',
                        'Method Not Allowed',
                        '/my_service.method1 is a ProtoRPC method.\n\n'
                        'Service %s.Service\n\n'
                        'More about ProtoRPC: '
                        'http://code.google.com/p/google-protorpc' %
                        (__name__,),
                        'text/plain; charset=utf-8')

    self.mox.VerifyAll()

  def testNoMatch_UnknownContentType(self):
    """Test what happens when no RPCMapper matches."""
    self.mox.ReplayAll()

    self.handler.request.headers['Content-Type'] = 'image/png'
    self.handler.handle('POST', '/my_service', 'method1')

    self.VerifyResponse('415',
                        'Unsupported content-type: image/png',
                        'Unsupported Media Type',
                        'text/plain; charset=utf-8')

    self.mox.VerifyAll()

  def testNoMatch_NoContentType(self):
    """Test what happens when no RPCMapper matches.."""
    self.mox.ReplayAll()

    self.handler.request.environ.pop('HTTP_CONTENT_TYPE', None)
    self.handler.request.headers.pop('Content-Type', None)
    self.handler.handle('/my_service', 'POST', 'method1')

    self.VerifyResponse('400', 'Invalid RPC request: missing content-type',
                        'Bad Request',
                        'text/plain; charset=utf-8')

    self.mox.VerifyAll()

  def testNoSuchMethod(self):
    """When service method not found."""
    self.ExpectRpcError(self.rpc_mapper1,
                        remote.RpcState.METHOD_NOT_FOUND_ERROR,
                        'Unrecognized RPC method: no_such_method')

    self.mox.ReplayAll()

    self.handler.handle('POST', '/my_service', 'no_such_method')

    self.VerifyResponse('400', 'Unrecognized RPC method: no_such_method', '')

    self.mox.VerifyAll()

  def testNoSuchRemoteMethod(self):
    """When service method exists but is not remote."""
    self.ExpectRpcError(self.rpc_mapper1,
                        remote.RpcState.METHOD_NOT_FOUND_ERROR,
                        'Unrecognized RPC method: not_remote')

    self.mox.ReplayAll()

    self.handler.handle('POST', '/my_service', 'not_remote')

    self.VerifyResponse('400', 'Unrecognized RPC method: not_remote', '')

    self.mox.VerifyAll()

  def testRequestError(self):
    """RequestError handling."""
    def build_request(handler, request):
      raise service_handlers.RequestError('This is a request error')
    self.rpc_mapper1.build_request(
        self.handler, Request1).WithSideEffects(build_request)

    self.ExpectRpcError(self.rpc_mapper1,
                        remote.RpcState.REQUEST_ERROR,
                        'Error parsing ProtoRPC request '
                        '(This is a request error)')

    self.mox.ReplayAll()

    self.handler.handle('POST', '/my_service', 'method1')

    self.VerifyResponse('400',
                        'Error parsing ProtoRPC request '
                        '(This is a request error)',
                        '')


    self.mox.VerifyAll()

  def testDecodeError(self):
    """DecodeError handling."""
    def build_request(handler, request):
      raise messages.DecodeError('This is a decode error')
    self.rpc_mapper1.build_request(
        self.handler, Request1).WithSideEffects(build_request)

    self.ExpectRpcError(self.rpc_mapper1,
                        remote.RpcState.REQUEST_ERROR,
                        r'Error parsing ProtoRPC request '
                        r'(This is a decode error)')

    self.mox.ReplayAll()

    self.handler.handle('POST', '/my_service', 'method1')

    self.VerifyResponse('400',
                        'Error parsing ProtoRPC request '
                        '(This is a decode error)',
                        '')

    self.mox.VerifyAll()

  def testResponseException(self):
    """Test what happens when build_response raises ResponseError."""
    self.rpc_mapper1.build_request(
        self.handler, Request1).AndReturn(self.request)

    self.rpc_mapper1.build_response(
        self.handler, mox.IsA(Response1)).AndRaise(
        service_handlers.ResponseError)

    self.ExpectRpcError(self.rpc_mapper1,
                        remote.RpcState.SERVER_ERROR,
                        'Internal Server Error')

    self.mox.ReplayAll()

    self.handler.handle('POST', '/my_service', 'method1')

    self.VerifyResponse('500', 'Internal Server Error', '')

    self.mox.VerifyAll()

  def testGet(self):
    """Test that GET goes to 'handle' properly."""
    self.handler.handle = self.mox.CreateMockAnything()
    self.handler.handle('GET', '/my_service', 'method1')
    self.handler.handle('GET', '/my_other_service', 'method2')

    self.mox.ReplayAll()

    self.handler.get('/my_service', 'method1')
    self.handler.get('/my_other_service', 'method2')

    self.mox.VerifyAll()

  def testPost(self):
    """Test that POST goes to 'handle' properly."""
    self.handler.handle = self.mox.CreateMockAnything()
    self.handler.handle('POST', '/my_service', 'method1')
    self.handler.handle('POST', '/my_other_service', 'method2')

    self.mox.ReplayAll()

    self.handler.post('/my_service', 'method1')
    self.handler.post('/my_other_service', 'method2')

    self.mox.VerifyAll()

  def testGetNoMethod(self):
    self.handler.get('/my_service', '')
    self.assertEquals(405, self.handler.response.status)
    self.assertEquals(
      util.pad_string('/my_service is a ProtoRPC service.\n\n'
                      'Service %s.Service\n\n'
                      'More about ProtoRPC: '
                      'http://code.google.com/p/google-protorpc\n' %
                      __name__),
      self.handler.response.out.getvalue())
    self.assertEquals(
        'nosniff',
        self.handler.response.headers['x-content-type-options'])

  def testGetNotSupported(self):
    self.handler.get('/my_service', 'method1')
    self.assertEquals(405, self.handler.response.status)
    expected_message = ('/my_service.method1 is a ProtoRPC method.\n\n'
                        'Service %s.Service\n\n'
                        'More about ProtoRPC: '
                        'http://code.google.com/p/google-protorpc\n' %
                        __name__)
    self.assertEquals(util.pad_string(expected_message),
                      self.handler.response.out.getvalue())
    self.assertEquals(
        'nosniff',
        self.handler.response.headers['x-content-type-options'])

  def testGetUnknownContentType(self):
    self.handler.request.headers['content-type'] = 'image/png'
    self.handler.get('/my_service', 'method1')
    self.assertEquals(415, self.handler.response.status)
    self.assertEquals(
      util.pad_string('/my_service.method1 is a ProtoRPC method.\n\n'
                      'Service %s.Service\n\n'
                      'More about ProtoRPC: '
                      'http://code.google.com/p/google-protorpc\n' %
                      __name__),
      self.handler.response.out.getvalue())
    self.assertEquals(
        'nosniff',
        self.handler.response.headers['x-content-type-options'])


class MissingContentLengthTests(ServiceHandlerTest):
  """Test for when content-length is not set in the environment.

  This test moves CONTENT_LENGTH from the environment to the
  content-length header.
  """

  def GetEnvironment(self):
    environment = super(MissingContentLengthTests, self).GetEnvironment()
    content_length = str(environment.pop('CONTENT_LENGTH', '0'))
    environment['HTTP_CONTENT_LENGTH'] = content_length
    return environment


class MissingContentTypeTests(ServiceHandlerTest):
  """Test for when content-type is not set in the environment.

  This test moves CONTENT_TYPE from the environment to the
  content-type header.
  """

  def GetEnvironment(self):
    environment = super(MissingContentTypeTests, self).GetEnvironment()
    content_type = str(environment.pop('CONTENT_TYPE', ''))
    environment['HTTP_CONTENT_TYPE'] = content_type
    return environment


class RPCMapperTestBase(test_util.TestCase):

  def setUp(self):
    """Set up test framework."""
    self.Reinitialize()

  def Reinitialize(self, input='',
                   get=False,
                   path_method='method1',
                   content_type='text/plain'):
    """Allows reinitialization of test with custom input values and POST.

    Args:
      input: Query string or POST input.
      get: Use GET method if True.  Use POST if False.
    """
    self.factory = service_handlers.ServiceHandlerFactory(Service)

    self.service_handler = service_handlers.ServiceHandler(self.factory,
                                                           Service())
    self.service_handler.remote_method = path_method
    request_path = '/servicepath'
    if path_method:
      request_path += '/' + path_method
    if get:
      request_path += '?' + input

    if get:
      environ = {'wsgi.input': cStringIO.StringIO(''),
                 'CONTENT_LENGTH': '0',
                 'QUERY_STRING': input,
                 'REQUEST_METHOD': 'GET',
                 'PATH_INFO': request_path,
                }
      self.service_handler.method = 'GET'
    else:
      environ = {'wsgi.input': cStringIO.StringIO(input),
                 'CONTENT_LENGTH': str(len(input)),
                 'QUERY_STRING': '',
                 'REQUEST_METHOD': 'POST',
                 'PATH_INFO': request_path,
                }
      self.service_handler.method = 'POST'

    self.request = webapp.Request(environ)

    self.response = webapp.Response()

    self.service_handler.initialize(self.request, self.response)

    self.service_handler.request.headers['Content-Type'] = content_type


class RPCMapperTest(RPCMapperTestBase, webapp_test_util.RequestHandlerTestBase):
  """Test the RPCMapper base class."""

  def setUp(self):
    RPCMapperTestBase.setUp(self)
    webapp_test_util.RequestHandlerTestBase.setUp(self)
    self.mox = mox.Mox()
    self.protocol = self.mox.CreateMockAnything()

  def GetEnvironment(self):
    """Get environment.

    Return bogus content in body.

    Returns:
      dict of CGI environment.
    """
    environment = super(RPCMapperTest, self).GetEnvironment()
    environment['wsgi.input'] = cStringIO.StringIO('my body')
    environment['CONTENT_LENGTH'] = len('my body')
    return environment

  def testContentTypes_JustDefault(self):
    """Test content type attributes."""
    self.mox.ReplayAll()

    mapper = service_handlers.RPCMapper(['GET', 'POST'],
                                        'my-content-type',
                                        self.protocol)

    self.assertEquals(frozenset(['GET', 'POST']), mapper.http_methods)
    self.assertEquals('my-content-type', mapper.default_content_type)
    self.assertEquals(frozenset(['my-content-type']),
                                mapper.content_types)

    self.mox.VerifyAll()

  def testContentTypes_Extended(self):
    """Test content type attributes."""
    self.mox.ReplayAll()

    mapper = service_handlers.RPCMapper(['GET', 'POST'],
                                        'my-content-type',
                                        self.protocol,
                                        content_types=['a', 'b'])

    self.assertEquals(frozenset(['GET', 'POST']), mapper.http_methods)
    self.assertEquals('my-content-type', mapper.default_content_type)
    self.assertEquals(frozenset(['my-content-type', 'a', 'b']),
                                mapper.content_types)

    self.mox.VerifyAll()

  def testBuildRequest(self):
    """Test building a request."""
    expected_request = Request1()
    self.protocol.decode_message(Request1,
                                 'my body').AndReturn(expected_request)

    self.mox.ReplayAll()

    mapper = service_handlers.RPCMapper(['POST'],
                                        'my-content-type',
                                        self.protocol)

    request = mapper.build_request(self.handler, Request1)

    self.assertTrue(expected_request is request)

  def testBuildRequest_ValidationError(self):
    """Test building a request generating a validation error."""
    expected_request = Request1()
    self.protocol.decode_message(
        Request1, 'my body').AndRaise(messages.ValidationError('xyz'))

    self.mox.ReplayAll()

    mapper = service_handlers.RPCMapper(['POST'],
                                        'my-content-type',
                                        self.protocol)

    self.assertRaisesWithRegexpMatch(
        service_handlers.RequestError,
        'Unable to parse request content: xyz',
        mapper.build_request,
        self.handler,
        Request1)

  def testBuildRequest_DecodeError(self):
    """Test building a request generating a decode error."""
    expected_request = Request1()
    self.protocol.decode_message(
        Request1, 'my body').AndRaise(messages.DecodeError('xyz'))

    self.mox.ReplayAll()

    mapper = service_handlers.RPCMapper(['POST'],
                                        'my-content-type',
                                        self.protocol)

    self.assertRaisesWithRegexpMatch(
        service_handlers.RequestError,
        'Unable to parse request content: xyz',
        mapper.build_request,
        self.handler,
        Request1)

  def testBuildResponse(self):
    """Test building a response."""
    response = Response1()
    self.protocol.encode_message(response).AndReturn('encoded')

    self.mox.ReplayAll()

    mapper = service_handlers.RPCMapper(['POST'],
                                        'my-content-type',
                                        self.protocol)

    request = mapper.build_response(self.handler, response)

    self.assertEquals('my-content-type',
                      self.handler.response.headers['Content-Type'])
    self.assertEquals('encoded', self.handler.response.out.getvalue())

  def testBuildResponse(self):
    """Test building a response."""
    response = Response1()
    self.protocol.encode_message(response).AndRaise(
        messages.ValidationError('xyz'))

    self.mox.ReplayAll()

    mapper = service_handlers.RPCMapper(['POST'],
                                        'my-content-type',
                                        self.protocol)

    self.assertRaisesWithRegexpMatch(service_handlers.ResponseError,
                                     'Unable to encode message: xyz',
                                     mapper.build_response,
                                     self.handler,
                                     response)


class ProtocolMapperTestBase(object):
  """Base class for basic protocol mapper tests."""

  def setUp(self):
    """Reinitialize test specifically for protocol buffer mapper."""
    super(ProtocolMapperTestBase, self).setUp()
    self.Reinitialize(path_method='my_method',
                      content_type='application/x-google-protobuf')

    self.request_message = Request1()
    self.request_message.integer_field = 1
    self.request_message.string_field = u'something'
    self.request_message.enum_field = Enum1.VAL1

    self.response_message = Response1()
    self.response_message.integer_field = 1
    self.response_message.string_field = u'something'
    self.response_message.enum_field = Enum1.VAL1

  def testBuildRequest(self):
    """Test request building."""
    self.Reinitialize(self.protocol.encode_message(self.request_message),
                      content_type=self.content_type)

    mapper = self.mapper()
    parsed_request = mapper.build_request(self.service_handler,
                                          Request1)
    self.assertEquals(self.request_message, parsed_request)

  def testBuildResponse(self):
    """Test response building."""

    mapper = self.mapper()
    mapper.build_response(self.service_handler, self.response_message)
    self.assertEquals(self.protocol.encode_message(self.response_message),
                      self.service_handler.response.out.getvalue())

  def testWholeRequest(self):
    """Test the basic flow of a request with mapper class."""
    body = self.protocol.encode_message(self.request_message)
    self.Reinitialize(input=body,
                      content_type=self.content_type)
    self.factory.add_request_mapper(self.mapper())
    self.service_handler.handle('POST', '/my_service', 'method1')
    VerifyResponse(self,
                   self.service_handler.response,
                   '200',
                   'OK',
                   self.protocol.encode_message(self.response_message),
                   self.content_type)


class URLEncodedRPCMapperTest(ProtocolMapperTestBase, RPCMapperTestBase):
  """Test the URL encoded RPC mapper."""

  content_type = 'application/x-www-form-urlencoded'
  protocol = protourlencode
  mapper = service_handlers.URLEncodedRPCMapper

  def testBuildRequest_Prefix(self):
    """Test building request with parameter prefix."""
    self.Reinitialize(urllib.urlencode([('prefix_integer_field', '10'),
                                        ('prefix_string_field', 'a string'),
                                        ('prefix_enum_field', 'VAL1'),
                                       ]),
                      self.content_type)

    url_encoded_mapper = service_handlers.URLEncodedRPCMapper(
        parameter_prefix='prefix_')
    request = url_encoded_mapper.build_request(self.service_handler,
                                               Request1)
    self.assertEquals(10, request.integer_field)
    self.assertEquals('a string', request.string_field)
    self.assertEquals(Enum1.VAL1, request.enum_field)

  def testBuildRequest_DecodeError(self):
    """Test trying to build request that causes a decode error."""
    self.Reinitialize(urllib.urlencode((('integer_field', '10'),
                                        ('integer_field', '20'),
                                        )),
                      content_type=self.content_type)

    url_encoded_mapper = service_handlers.URLEncodedRPCMapper()

    self.assertRaises(service_handlers.RequestError,
                      url_encoded_mapper.build_request,
                      self.service_handler,
                      Service.method1.remote.request_type)

  def testBuildResponse_Prefix(self):
    """Test building a response with parameter prefix."""
    response = Response1()
    response.integer_field = 10
    response.string_field = u'a string'
    response.enum_field = Enum1.VAL3

    url_encoded_mapper = service_handlers.URLEncodedRPCMapper(
        parameter_prefix='prefix_')

    url_encoded_mapper.build_response(self.service_handler, response)
    self.assertEquals('application/x-www-form-urlencoded',
                      self.response.headers['content-type'])
    self.assertEquals(cgi.parse_qs(self.response.out.getvalue(), True, True),
                      {'prefix_integer_field': ['10'],
                       'prefix_string_field': [u'a string'],
                       'prefix_enum_field': ['VAL3'],
                      })


class ProtobufRPCMapperTest(ProtocolMapperTestBase, RPCMapperTestBase):
  """Test the protobuf encoded RPC mapper."""

  content_type = 'application/octet-stream'
  protocol = protobuf
  mapper = service_handlers.ProtobufRPCMapper


class JSONRPCMapperTest(ProtocolMapperTestBase, RPCMapperTestBase):
  """Test the URL encoded RPC mapper."""

  content_type = 'application/json'
  protocol = protojson
  mapper = service_handlers.JSONRPCMapper


class MyService(remote.Service):

  def __init__(self, value='default'):
    self.value = value


class ServiceMappingTest(test_util.TestCase):

  def CheckFormMappings(self, mapping, registry_path='/protorpc'):
    """Check to make sure that form mapping is configured as expected.

    Args:
      mapping: Mapping that should contain forms handlers.
    """
    pattern, factory = mapping[0]
    self.assertEquals('%s/form(?:/)?' % registry_path, pattern)
    handler = factory()
    self.assertTrue(isinstance(handler, forms.FormsHandler))
    self.assertEquals(registry_path, handler.registry_path)

    pattern, factory = mapping[1]
    self.assertEquals('%s/form/(.+)' % registry_path, pattern)
    self.assertEquals(forms.ResourceHandler, factory)


  def DoMappingTest(self,
                    services,
                    registry_path='/myreg',
                    expected_paths=None):
    mapped_services = mapping = service_handlers.service_mapping(services,
                                                                 registry_path)
    if registry_path:
      form_mapping = mapping[:2]
      mapped_registry_path, mapped_registry_factory = mapping[-1]
      mapped_services = mapping[2:-1]
      self.CheckFormMappings(form_mapping, registry_path=registry_path)

      self.assertEquals(r'(%s)%s' % (registry_path,
                                     service_handlers._METHOD_PATTERN),
                        mapped_registry_path)
      self.assertEquals(registry.RegistryService,
                        mapped_registry_factory.service_factory.service_class)

      # Verify registry knows about other services.
      expected_registry = {registry_path: registry.RegistryService}
      for path, factory in dict(services).items():
        if isinstance(factory, type) and issubclass(factory, remote.Service):
          expected_registry[path] = factory
        else:
          expected_registry[path] = factory.service_class
      self.assertEquals(expected_registry,
                        mapped_registry_factory().service.registry)

    # Verify that services are mapped to URL.
    self.assertEquals(len(services), len(mapped_services))
    for path, service in dict(services).items():
      mapped_path = r'(%s)%s' %  (path, service_handlers._METHOD_PATTERN)
      mapped_factory = dict(mapped_services)[mapped_path]
      self.assertEquals(service, mapped_factory.service_factory)

  def testServiceMapping_Empty(self):
    """Test an empty service mapping."""
    self.DoMappingTest({})

  def testServiceMapping_ByClass(self):
    """Test mapping a service by class."""
    self.DoMappingTest({'/my-service': MyService})

  def testServiceMapping_ByFactory(self):
    """Test mapping a service by factory."""
    self.DoMappingTest({'/my-service': MyService.new_factory('new-value')})

  def testServiceMapping_ByList(self):
    """Test mapping a service by factory."""
    self.DoMappingTest(
      [('/my-service1', MyService.new_factory('service1')),
       ('/my-service2', MyService.new_factory('service2')),
      ])

  def testServiceMapping_NoRegistry(self):
    """Test mapping a service by class."""
    mapping = self.DoMappingTest({'/my-service': MyService}, None)

  def testDefaultMappingWithClass(self):
    """Test setting path just from the class.

    Path of the mapping will be the fully qualified ProtoRPC service name with
    '.' replaced with '/'.  For example:

      com.nowhere.service.TheService -> /com/nowhere/service/TheService
    """
    mapping = service_handlers.service_mapping([MyService])
    mapped_services = mapping[2:-1]
    self.assertEquals(1, len(mapped_services))
    path, factory = mapped_services[0]

    self.assertEquals(
      r'(/test_package/MyService)' + service_handlers._METHOD_PATTERN,
      path)
    self.assertEquals(MyService, factory.service_factory)

  def testDefaultMappingWithFactory(self):
    mapping = service_handlers.service_mapping(
      [MyService.new_factory('service1')])
    mapped_services = mapping[2:-1]
    self.assertEquals(1, len(mapped_services))
    path, factory = mapped_services[0]

    self.assertEquals(
      r'(/test_package/MyService)' + service_handlers._METHOD_PATTERN,
      path)
    self.assertEquals(MyService, factory.service_factory.service_class)

  def testMappingDuplicateExplicitServiceName(self):
    self.assertRaisesWithRegexpMatch(
      service_handlers.ServiceConfigurationError,
      "Path '/my_path' is already defined in service mapping",
      service_handlers.service_mapping,
      [('/my_path', MyService),
       ('/my_path', MyService),
       ])

  def testMappingDuplicateServiceName(self):
    self.assertRaisesWithRegexpMatch(
      service_handlers.ServiceConfigurationError,
      "Path '/test_package/MyService' is already defined in service mapping",
      service_handlers.service_mapping,
      [MyService, MyService])


class GetCalled(remote.Service):

  def __init__(self, test):
    self.test = test

  @remote.method(Request1, Response1)
  def my_method(self, request):
    self.test.request = request
    return Response1(string_field='a response')


class TestRunServices(test_util.TestCase):

  def DoRequest(self,
                path,
                request,
                response_type,
                reg_path='/protorpc'):
    stdin = sys.stdin
    stdout = sys.stdout
    environ = os.environ
    try:
      sys.stdin = cStringIO.StringIO(protojson.encode_message(request))
      sys.stdout = cStringIO.StringIO()

      os.environ = webapp_test_util.GetDefaultEnvironment()
      os.environ['PATH_INFO'] = path
      os.environ['REQUEST_METHOD'] = 'POST'
      os.environ['CONTENT_TYPE'] = 'application/json'
      os.environ['wsgi.input'] = sys.stdin
      os.environ['wsgi.output'] = sys.stdout
      os.environ['CONTENT_LENGTH'] = len(sys.stdin.getvalue())

      service_handlers.run_services(
        [('/my_service', GetCalled.new_factory(self))], reg_path)

      header, body = sys.stdout.getvalue().split('\n\n', 1)

      return (header.split('\n')[0],
              protojson.decode_message(response_type, body))
    finally:
      sys.stdin = stdin
      sys.stdout = stdout
      os.environ = environ

  def testRequest(self):
    request = Request1(string_field='request value')

    status, response = self.DoRequest('/my_service.my_method',
                                      request,
                                      Response1)
    self.assertEquals('Status: 200 OK', status)
    self.assertEquals(request, self.request)
    self.assertEquals(Response1(string_field='a response'), response)

  def testRegistry(self):
    request = Request1(string_field='request value')
    status, response = self.DoRequest('/protorpc.services',
                              message_types.VoidMessage(),
                              registry.ServicesResponse)

    self.assertEquals('Status: 200 OK', status)
    self.assertIterEqual([
        registry.ServiceMapping(
            name='/protorpc',
            definition='protorpc.registry.RegistryService'),
        registry.ServiceMapping(
            name='/my_service',
            definition='test_package.GetCalled'),
        ], response.services)

  def testRunServicesWithOutRegistry(self):
    request = Request1(string_field='request value')

    status, response = self.DoRequest('/protorpc.services',
                                      message_types.VoidMessage(),
                                      registry.ServicesResponse,
                                      reg_path=None)
    self.assertEquals('Status: 404 Not Found', status)


def main():
  unittest.main()


if __name__ == '__main__':
  main()



#!/usr/bin/env python
#
# Copyright 2010 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""Tests for protorpc.forms."""

__author__ = 'rafek@google.com (Rafe Kaplan)'


import os
import unittest

from protorpc import test_util
from protorpc import webapp_test_util
from protorpc.webapp import forms
from protorpc.webapp.google_imports import template


class ModuleInterfaceTest(test_util.ModuleInterfaceTest,
                          test_util.TestCase):

  MODULE = forms


def RenderTemplate(name, **params):
  """Load content from static file.

  Args:
    name: Name of static file to load from static directory.
    params: Passed in to webapp template generator.

  Returns:
    Contents of static file.
  """
  path = os.path.join(forms._TEMPLATES_DIR, name)
  return template.render(path, params)


class ResourceHandlerTest(webapp_test_util.RequestHandlerTestBase):

  def CreateRequestHandler(self):
    return forms.ResourceHandler()

  def DoStaticContentTest(self, name, expected_type):
    """Run the static content test.

    Loads expected static content from source and compares with
    results in response.  Checks content-type and cache header.

    Args:
      name: Name of file that should be served.
      expected_type: Expected content-type of served file.
    """
    self.handler.get(name)

    content = RenderTemplate(name)
    self.CheckResponse('200 OK',
                       {'content-type': expected_type,
                       },
                       content)

  def testGet(self):
    self.DoStaticContentTest('forms.js', 'text/javascript')

  def testNoSuchFile(self):
    self.handler.get('unknown.txt')

    self.CheckResponse('404 Not Found',
                       {},
                       'Resource not found.')


class FormsHandlerTest(webapp_test_util.RequestHandlerTestBase):

  def CreateRequestHandler(self):
    handler = forms.FormsHandler('/myreg')
    self.assertEquals('/myreg', handler.registry_path)
    return handler

  def testGetForm(self):
    self.handler.get()

    content = RenderTemplate(
        'forms.html',
        forms_path='/tmp/myhandler',
        hostname=self.request.host,
        registry_path='/myreg')

    self.CheckResponse('200 OK',
                       {},
                       content)

  def testGet_MissingPath(self):
    self.ResetHandler({'QUERY_STRING': 'method=my_method'})

    self.handler.get()

    content = RenderTemplate(
        'forms.html',
        forms_path='/tmp/myhandler',
        hostname=self.request.host,
        registry_path='/myreg')

    self.CheckResponse('200 OK',
                       {},
                       content)

  def testGet_MissingMethod(self):
    self.ResetHandler({'QUERY_STRING': 'path=/my-path'})

    self.handler.get()

    content = RenderTemplate(
        'forms.html',
        forms_path='/tmp/myhandler',
        hostname=self.request.host,
        registry_path='/myreg')

    self.CheckResponse('200 OK',
                       {},
                       content)

  def testGetMethod(self):
    self.ResetHandler({'QUERY_STRING': 'path=/my-path&method=my_method'})

    self.handler.get()

    content = RenderTemplate(
        'methods.html',
        forms_path='/tmp/myhandler',
        hostname=self.request.host,
        registry_path='/myreg',
        service_path='/my-path',
        method_name='my_method')

    self.CheckResponse('200 OK',
                       {},
                       content)


def main():
  unittest.main()


if __name__ == '__main__':
  main()



#!/usr/bin/env python
#
# Copyright 2011 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""WSGI utilities

Small collection of helpful utilities for working with WSGI.
"""
import six

__author__ = 'rafek@google.com (Rafe Kaplan)'

import six.moves.http_client
import re

from .. import util

__all__ = ['static_page',
           'error',
           'first_found',
]

_STATUS_PATTERN = re.compile('^(\d{3})\s')


@util.positional(1)
def static_page(content='',
                status='200 OK',
                content_type='text/html; charset=utf-8',
                headers=None):
  """Create a WSGI application that serves static content.

  A static page is one that will be the same every time it receives a request.
  It will always serve the same status, content and headers.

  Args:
    content: Content to serve in response to HTTP request.
    status: Status to serve in response to HTTP request.  If string, status
      is served as is without any error checking.  If integer, will look up
      status message.  Otherwise, parameter is tuple (status, description):
        status: Integer status of response.
        description: Brief text description of response.
    content_type: Convenient parameter for content-type header.  Will appear
      before any content-type header that appears in 'headers' parameter.
    headers: Dictionary of headers or iterable of tuples (name, value):
      name: String name of header.
      value: String value of header.

  Returns:
    WSGI application that serves static content.
  """
  if isinstance(status, six.integer_types):
    status = '%d %s' % (status, six.moves.http_client.responses.get(status, 'Unknown Error'))
  elif not isinstance(status, six.string_types):
    status = '%d %s' % tuple(status)

  if isinstance(headers, dict):
    headers = six.iteritems(headers)

  headers = [('content-length', str(len(content))),
             ('content-type', content_type),
            ] + list(headers or [])

  # Ensure all headers are str.
  for index, (key, value) in enumerate(headers):
    try:
      value = value.decode('utf-8')
      headers[index] = key, value
    except:
      pass

    if not isinstance(key, str):
      raise TypeError('Header key must be str, found: %r' % (key,))

    if not isinstance(value, str):
      raise TypeError(
          'Header %r must be type str or unicode, found: %r' % (key, value))

  def static_page_application(environ, start_response):
    start_response(status, headers)
    return [content]

  return static_page_application


@util.positional(2)
def error(status_code, status_message=None,
          content_type='text/plain; charset=utf-8',
          headers=None, content=None):
  """Create WSGI application that statically serves an error page.

  Creates a static error page specifically for non-200 HTTP responses.

  Browsers such as Internet Explorer will display their own error pages for
  error content responses smaller than 512 bytes.  For this reason all responses
  are right-padded up to 512 bytes.

  Error pages that are not provided will content will contain the standard HTTP
  status message as their content.

  Args:
    status_code: Integer status code of error.
    status_message: Status message.

  Returns:
    Static WSGI application that sends static error response.
  """
  if status_message is None:
    status_message = six.moves.http_client.responses.get(status_code, 'Unknown Error')

  if content is None:
    content = status_message

  content = util.pad_string(content)

  return static_page(content,
                     status=(status_code, status_message),
                     content_type=content_type,
                     headers=headers)


def first_found(apps):
  """Serve the first application that does not response with 404 Not Found.

  If no application serves content, will respond with generic 404 Not Found.

  Args:
    apps: List of WSGI applications to search through.  Will serve the content
      of the first of these that does not return a 404 Not Found.  Applications
      in this list must not modify the environment or any objects in it if they
      do not match.  Applications that do not obey this restriction can create
      unpredictable results.

  Returns:
    Compound application that serves the contents of the first application that
    does not response with 404 Not Found.
  """
  apps = tuple(apps)
  not_found = error(six.moves.http_client.NOT_FOUND)

  def first_found_app(environ, start_response):
    """Compound application returned from the first_found function."""
    final_result = {}  # Used in absence of Python local scoping.

    def first_found_start_response(status, response_headers):
      """Replacement for start_response as passed in to first_found_app.

      Called by each application in apps instead of the real start response.
      Checks the response status, and if anything other than 404, sets 'status'
      and 'response_headers' in final_result.
      """
      status_match = _STATUS_PATTERN.match(status)
      assert status_match, ('Status must be a string beginning '
                            'with 3 digit number. Found: %s' % status)
      status_code = status_match.group(0)
      if int(status_code) == six.moves.http_client.NOT_FOUND:
        return

      final_result['status'] = status
      final_result['response_headers'] = response_headers

    for app in apps:
      response = app(environ, first_found_start_response)
      if final_result:
        start_response(final_result['status'], final_result['response_headers'])
        return response

    return not_found(environ, start_response)
  return first_found_app



#!/usr/bin/env python
#
# Copyright 2010 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""Tests for protorpc.protourlencode."""

__author__ = 'rafek@google.com (Rafe Kaplan)'


import cgi
import logging
import unittest
import urllib

from protorpc import message_types
from protorpc import messages
from protorpc import protourlencode
from protorpc import test_util


class ModuleInterfaceTest(test_util.ModuleInterfaceTest,
                          test_util.TestCase):

  MODULE = protourlencode



class SuperMessage(messages.Message):
  """A test message with a nested message field."""

  sub_message = messages.MessageField(test_util.OptionalMessage, 1)
  sub_messages = messages.MessageField(test_util.OptionalMessage,
                                       2,
                                       repeated=True)


class SuperSuperMessage(messages.Message):
  """A test message with two levels of nested."""

  sub_message = messages.MessageField(SuperMessage, 1)
  sub_messages = messages.MessageField(SuperMessage, 2, repeated=True)


class URLEncodedRequestBuilderTest(test_util.TestCase):
  """Test the URL Encoded request builder."""

  def testMakePath(self):
    builder = protourlencode.URLEncodedRequestBuilder(SuperSuperMessage(),
                                                      prefix='pre.')

    self.assertEquals(None, builder.make_path(''))
    self.assertEquals(None, builder.make_path('no_such_field'))
    self.assertEquals(None, builder.make_path('pre.no_such_field'))

    # Missing prefix.
    self.assertEquals(None, builder.make_path('sub_message'))

    # Valid parameters.
    self.assertEquals((('sub_message', None),),
                       builder.make_path('pre.sub_message'))
    self.assertEquals((('sub_message', None), ('sub_messages', 1)),
                       builder.make_path('pre.sub_message.sub_messages-1'))
    self.assertEquals(
        (('sub_message', None),
         ('sub_messages', 1),
         ('int64_value', None)),
        builder.make_path('pre.sub_message.sub_messages-1.int64_value'))

    # Missing index.
    self.assertEquals(
        None,
        builder.make_path('pre.sub_message.sub_messages.integer_field'))

    # Has unexpected index.
    self.assertEquals(
        None,
        builder.make_path('pre.sub_message.sub_message-1.integer_field'))

  def testAddParameter_SimpleAttributes(self):
    message = test_util.OptionalMessage()
    builder = protourlencode.URLEncodedRequestBuilder(message, prefix='pre.')

    self.assertTrue(builder.add_parameter('pre.int64_value', ['10']))
    self.assertTrue(builder.add_parameter('pre.string_value', ['a string']))
    self.assertTrue(builder.add_parameter('pre.enum_value', ['VAL1']))
    self.assertEquals(10, message.int64_value)
    self.assertEquals('a string', message.string_value)
    self.assertEquals(test_util.OptionalMessage.SimpleEnum.VAL1,
                      message.enum_value)

  def testAddParameter_InvalidAttributes(self):
    message = SuperSuperMessage()
    builder = protourlencode.URLEncodedRequestBuilder(message, prefix='pre.')

    def assert_empty():
      self.assertEquals(None, getattr(message, 'sub_message'))
      self.assertEquals([], getattr(message, 'sub_messages'))

    self.assertFalse(builder.add_parameter('pre.nothing', ['x']))
    assert_empty()

    self.assertFalse(builder.add_parameter('pre.sub_messages', ['x']))
    self.assertFalse(builder.add_parameter('pre.sub_messages-1.nothing', ['x']))
    assert_empty()

  def testAddParameter_NestedAttributes(self):
    message = SuperSuperMessage()
    builder = protourlencode.URLEncodedRequestBuilder(message, prefix='pre.')

    # Set an empty message fields.
    self.assertTrue(builder.add_parameter('pre.sub_message', ['']))
    self.assertTrue(isinstance(message.sub_message, SuperMessage))

    # Add a basic attribute.
    self.assertTrue(builder.add_parameter(
        'pre.sub_message.sub_message.int64_value', ['10']))
    self.assertTrue(builder.add_parameter(
        'pre.sub_message.sub_message.string_value', ['hello']))

    self.assertTrue(10, message.sub_message.sub_message.int64_value)
    self.assertTrue('hello', message.sub_message.sub_message.string_value)


  def testAddParameter_NestedMessages(self):
    message = SuperSuperMessage()
    builder = protourlencode.URLEncodedRequestBuilder(message, prefix='pre.')

    # Add a repeated empty message.
    self.assertTrue(builder.add_parameter(
        'pre.sub_message.sub_messages-0', ['']))
    sub_message = message.sub_message.sub_messages[0]
    self.assertTrue(1, len(message.sub_message.sub_messages))
    self.assertTrue(isinstance(sub_message,
                               test_util.OptionalMessage))
    self.assertEquals(None, getattr(sub_message, 'int64_value'))
    self.assertEquals(None, getattr(sub_message, 'string_value'))
    self.assertEquals(None, getattr(sub_message, 'enum_value'))

    # Add a repeated message with value.
    self.assertTrue(builder.add_parameter(
        'pre.sub_message.sub_messages-1.int64_value', ['10']))
    self.assertTrue(2, len(message.sub_message.sub_messages))
    self.assertTrue(10, message.sub_message.sub_messages[1].int64_value)

    # Add another value to the same nested message.
    self.assertTrue(builder.add_parameter(
        'pre.sub_message.sub_messages-1.string_value', ['a string']))
    self.assertTrue(2, len(message.sub_message.sub_messages))
    self.assertEquals(10, message.sub_message.sub_messages[1].int64_value)
    self.assertEquals('a string',
                      message.sub_message.sub_messages[1].string_value)

  def testAddParameter_RepeatedValues(self):
    message = test_util.RepeatedMessage()
    builder = protourlencode.URLEncodedRequestBuilder(message, prefix='pre.')

    self.assertTrue(builder.add_parameter('pre.int64_value-0', ['20']))
    self.assertTrue(builder.add_parameter('pre.int64_value-1', ['30']))
    self.assertEquals([20, 30], message.int64_value)

    self.assertTrue(builder.add_parameter('pre.string_value-0', ['hi']))
    self.assertTrue(builder.add_parameter('pre.string_value-1', ['lo']))
    self.assertTrue(builder.add_parameter('pre.string_value-1', ['dups overwrite']))
    self.assertEquals(['hi', 'dups overwrite'], message.string_value)

  def testAddParameter_InvalidValuesMayRepeat(self):
    message = test_util.OptionalMessage()
    builder = protourlencode.URLEncodedRequestBuilder(message, prefix='pre.')

    self.assertFalse(builder.add_parameter('nothing', [1, 2, 3]))

  def testAddParameter_RepeatedParameters(self):
    message = test_util.OptionalMessage()
    builder = protourlencode.URLEncodedRequestBuilder(message, prefix='pre.')

    self.assertRaises(messages.DecodeError,
                      builder.add_parameter,
                      'pre.int64_value',
                      [1, 2, 3])
    self.assertRaises(messages.DecodeError,
                      builder.add_parameter,
                      'pre.int64_value',
                      [])

  def testAddParameter_UnexpectedNestedValue(self):
    """Test getting a nested value on a non-message sub-field."""
    message = test_util.HasNestedMessage()
    builder = protourlencode.URLEncodedRequestBuilder(message, 'pre.')

    self.assertFalse(builder.add_parameter('pre.nested.a_value.whatever',
                                           ['1']))

  def testInvalidFieldFormat(self):
    message = test_util.OptionalMessage()
    builder = protourlencode.URLEncodedRequestBuilder(message, prefix='pre.')

    self.assertFalse(builder.add_parameter('pre.illegal%20', ['1']))

  def testAddParameter_UnexpectedNestedValue(self):
    """Test getting a nested value on a non-message sub-field

    There is an odd corner case where if trying to insert a repeated value
    on an nested repeated message that would normally succeed in being created
    should fail.  This case can only be tested when the first message of the
    nested messages already exists.

    Another case is trying to access an indexed value nested within a
    non-message field.
    """
    class HasRepeated(messages.Message):

      values = messages.IntegerField(1, repeated=True)

    class HasNestedRepeated(messages.Message):

      nested = messages.MessageField(HasRepeated, 1, repeated=True)


    message = HasNestedRepeated()
    builder = protourlencode.URLEncodedRequestBuilder(message, prefix='pre.')

    self.assertTrue(builder.add_parameter('pre.nested-0.values-0', ['1']))
    # Try to create an indexed value on a non-message field.
    self.assertFalse(builder.add_parameter('pre.nested-0.values-0.unknown-0',
                                           ['1']))
    # Try to create an out of range indexed field on an otherwise valid
    # repeated message field.
    self.assertFalse(builder.add_parameter('pre.nested-1.values-1', ['1']))


class ProtourlencodeConformanceTest(test_util.TestCase,
                                    test_util.ProtoConformanceTestBase):

  PROTOLIB = protourlencode

  encoded_partial = urllib.urlencode([('double_value', 1.23),
                                      ('int64_value', -100000000000),
                                      ('int32_value', 1020),
                                      ('string_value', u'a string'),
                                      ('enum_value', 'VAL2'),
                                     ])

  encoded_full = urllib.urlencode([('double_value', 1.23),
                                   ('float_value', -2.5),
                                   ('int64_value', -100000000000),
                                   ('uint64_value', 102020202020),
                                   ('int32_value', 1020),
                                   ('bool_value', 'true'),
                                   ('string_value',
                                    u'a string\u044f'.encode('utf-8')),
                                   ('bytes_value', b'a bytes\xff\xfe'),
                                   ('enum_value', 'VAL2'),
                                  ])

  encoded_repeated = urllib.urlencode([('double_value-0', 1.23),
                                       ('double_value-1', 2.3),
                                       ('float_value-0', -2.5),
                                       ('float_value-1', 0.5),
                                       ('int64_value-0', -100000000000),
                                       ('int64_value-1', 20),
                                       ('uint64_value-0', 102020202020),
                                       ('uint64_value-1', 10),
                                       ('int32_value-0', 1020),
                                       ('int32_value-1', 718),
                                       ('bool_value-0', 'true'),
                                       ('bool_value-1', 'false'),
                                       ('string_value-0',
                                        u'a string\u044f'.encode('utf-8')),
                                       ('string_value-1',
                                        u'another string'.encode('utf-8')),
                                       ('bytes_value-0', b'a bytes\xff\xfe'),
                                       ('bytes_value-1', b'another bytes'),
                                       ('enum_value-0', 'VAL2'),
                                       ('enum_value-1', 'VAL1'),
                                      ])

  encoded_nested = urllib.urlencode([('nested.a_value', 'a string'),
                                    ])

  encoded_repeated_nested = urllib.urlencode(
      [('repeated_nested-0.a_value', 'a string'),
       ('repeated_nested-1.a_value', 'another string'),
       ])

  unexpected_tag_message = 'unexpected=whatever'

  encoded_default_assigned = urllib.urlencode([('a_value', 'a default'),
                                              ])

  encoded_nested_empty = urllib.urlencode([('nested', '')])

  encoded_repeated_nested_empty = urllib.urlencode([('repeated_nested-0', ''),
                                                    ('repeated_nested-1', '')])

  encoded_extend_message = urllib.urlencode([('int64_value-0', 400),
                                             ('int64_value-1', 50),
                                             ('int64_value-2', 6000)])

  encoded_string_types = urllib.urlencode(
    [('string_value', 'Latin')])

  encoded_invalid_enum = urllib.urlencode([('enum_value', 'undefined')])

  def testParameterPrefix(self):
    """Test using the 'prefix' parameter to encode_message."""
    class MyMessage(messages.Message):
      number = messages.IntegerField(1)
      names = messages.StringField(2, repeated=True)

    message = MyMessage()
    message.number = 10
    message.names = [u'Fred', u'Lisa']

    encoded_message = protourlencode.encode_message(message, prefix='prefix-')
    self.assertEquals({'prefix-number': ['10'],
                       'prefix-names-0': ['Fred'],
                       'prefix-names-1': ['Lisa'],
                      },
                      cgi.parse_qs(encoded_message))

    self.assertEquals(message, protourlencode.decode_message(MyMessage,
                                                             encoded_message,
                                                             prefix='prefix-'))

  def testProtourlencodeUnrecognizedField(self):
    """Test that unrecognized fields are saved and can be accessed."""

    class MyMessage(messages.Message):
      number = messages.IntegerField(1)

    decoded = protourlencode.decode_message(MyMessage,
                                            self.unexpected_tag_message)
    self.assertEquals(1, len(decoded.all_unrecognized_fields()))
    self.assertEquals('unexpected', decoded.all_unrecognized_fields()[0])
    # Unknown values set to a list of however many values had that name.
    self.assertEquals((['whatever'], messages.Variant.STRING),
                      decoded.get_unrecognized_field_info('unexpected'))

    repeated_unknown = urllib.urlencode([('repeated', 400),
                                         ('repeated', 'test'),
                                         ('repeated', '123.456')])
    decoded2 = protourlencode.decode_message(MyMessage, repeated_unknown)
    self.assertEquals((['400', 'test', '123.456'], messages.Variant.STRING),
                      decoded2.get_unrecognized_field_info('repeated'))

  def testDecodeInvalidDateTime(self):

    class MyMessage(messages.Message):
      a_datetime = message_types.DateTimeField(1)

    self.assertRaises(messages.DecodeError, protourlencode.decode_message,
                      MyMessage, 'a_datetime=invalid')


if __name__ == '__main__':
  unittest.main()



# Generated by the protocol buffer compiler.  DO NOT EDIT (except the imports)!

# Replace auto generated imports with .non_sdk_imports manually!
# Do the replacement and copy this comment everytime!
from .non_sdk_imports import descriptor
from .non_sdk_imports import message
from .non_sdk_imports import reflection
from .non_sdk_imports import descriptor_pb2
import six

# @@protoc_insertion_point(imports)



DESCRIPTOR = descriptor.FileDescriptor(
  name='protorpc_test.proto',
  package='protorpc',
  serialized_pb='\n\x13protorpc_test.proto\x12\x08protorpc\" \n\rNestedMessage\x12\x0f\n\x07\x61_value\x18\x01 \x02(\t\"m\n\x10HasNestedMessage\x12\'\n\x06nested\x18\x01 \x01(\x0b\x32\x17.protorpc.NestedMessage\x12\x30\n\x0frepeated_nested\x18\x02 \x03(\x0b\x32\x17.protorpc.NestedMessage\"(\n\nHasDefault\x12\x1a\n\x07\x61_value\x18\x01 \x01(\t:\ta default\"\x97\x02\n\x0fOptionalMessage\x12\x14\n\x0c\x64ouble_value\x18\x01 \x01(\x01\x12\x13\n\x0b\x66loat_value\x18\x02 \x01(\x02\x12\x13\n\x0bint64_value\x18\x03 \x01(\x03\x12\x14\n\x0cuint64_value\x18\x04 \x01(\x04\x12\x13\n\x0bint32_value\x18\x05 \x01(\x05\x12\x12\n\nbool_value\x18\x06 \x01(\x08\x12\x14\n\x0cstring_value\x18\x07 \x01(\t\x12\x13\n\x0b\x62ytes_value\x18\x08 \x01(\x0c\x12\x38\n\nenum_value\x18\n \x01(\x0e\x32$.protorpc.OptionalMessage.SimpleEnum\" \n\nSimpleEnum\x12\x08\n\x04VAL1\x10\x01\x12\x08\n\x04VAL2\x10\x02\"\x97\x02\n\x0fRepeatedMessage\x12\x14\n\x0c\x64ouble_value\x18\x01 \x03(\x01\x12\x13\n\x0b\x66loat_value\x18\x02 \x03(\x02\x12\x13\n\x0bint64_value\x18\x03 \x03(\x03\x12\x14\n\x0cuint64_value\x18\x04 \x03(\x04\x12\x13\n\x0bint32_value\x18\x05 \x03(\x05\x12\x12\n\nbool_value\x18\x06 \x03(\x08\x12\x14\n\x0cstring_value\x18\x07 \x03(\t\x12\x13\n\x0b\x62ytes_value\x18\x08 \x03(\x0c\x12\x38\n\nenum_value\x18\n \x03(\x0e\x32$.protorpc.RepeatedMessage.SimpleEnum\" \n\nSimpleEnum\x12\x08\n\x04VAL1\x10\x01\x12\x08\n\x04VAL2\x10\x02\"y\n\x18HasOptionalNestedMessage\x12)\n\x06nested\x18\x01 \x01(\x0b\x32\x19.protorpc.OptionalMessage\x12\x32\n\x0frepeated_nested\x18\x02 \x03(\x0b\x32\x19.protorpc.OptionalMessage')



_OPTIONALMESSAGE_SIMPLEENUM = descriptor.EnumDescriptor(
  name='SimpleEnum',
  full_name='protorpc.OptionalMessage.SimpleEnum',
  filename=None,
  file=DESCRIPTOR,
  values=[
    descriptor.EnumValueDescriptor(
      name='VAL1', index=0, number=1,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='VAL2', index=1, number=2,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=468,
  serialized_end=500,
)

_REPEATEDMESSAGE_SIMPLEENUM = descriptor.EnumDescriptor(
  name='SimpleEnum',
  full_name='protorpc.RepeatedMessage.SimpleEnum',
  filename=None,
  file=DESCRIPTOR,
  values=[
    descriptor.EnumValueDescriptor(
      name='VAL1', index=0, number=1,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='VAL2', index=1, number=2,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=468,
  serialized_end=500,
)


_NESTEDMESSAGE = descriptor.Descriptor(
  name='NestedMessage',
  full_name='protorpc.NestedMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='a_value', full_name='protorpc.NestedMessage.a_value', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=six.text_type("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=33,
  serialized_end=65,
)


_HASNESTEDMESSAGE = descriptor.Descriptor(
  name='HasNestedMessage',
  full_name='protorpc.HasNestedMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='nested', full_name='protorpc.HasNestedMessage.nested', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='repeated_nested', full_name='protorpc.HasNestedMessage.repeated_nested', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=67,
  serialized_end=176,
)


_HASDEFAULT = descriptor.Descriptor(
  name='HasDefault',
  full_name='protorpc.HasDefault',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='a_value', full_name='protorpc.HasDefault.a_value', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=True, default_value=six.text_type("a default", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=178,
  serialized_end=218,
)


_OPTIONALMESSAGE = descriptor.Descriptor(
  name='OptionalMessage',
  full_name='protorpc.OptionalMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='double_value', full_name='protorpc.OptionalMessage.double_value', index=0,
      number=1, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='float_value', full_name='protorpc.OptionalMessage.float_value', index=1,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='int64_value', full_name='protorpc.OptionalMessage.int64_value', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='uint64_value', full_name='protorpc.OptionalMessage.uint64_value', index=3,
      number=4, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='int32_value', full_name='protorpc.OptionalMessage.int32_value', index=4,
      number=5, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='bool_value', full_name='protorpc.OptionalMessage.bool_value', index=5,
      number=6, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='string_value', full_name='protorpc.OptionalMessage.string_value', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=six.text_type("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='bytes_value', full_name='protorpc.OptionalMessage.bytes_value', index=7,
      number=8, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value="",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='enum_value', full_name='protorpc.OptionalMessage.enum_value', index=8,
      number=10, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _OPTIONALMESSAGE_SIMPLEENUM,
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=221,
  serialized_end=500,
)


_REPEATEDMESSAGE = descriptor.Descriptor(
  name='RepeatedMessage',
  full_name='protorpc.RepeatedMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='double_value', full_name='protorpc.RepeatedMessage.double_value', index=0,
      number=1, type=1, cpp_type=5, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='float_value', full_name='protorpc.RepeatedMessage.float_value', index=1,
      number=2, type=2, cpp_type=6, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='int64_value', full_name='protorpc.RepeatedMessage.int64_value', index=2,
      number=3, type=3, cpp_type=2, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='uint64_value', full_name='protorpc.RepeatedMessage.uint64_value', index=3,
      number=4, type=4, cpp_type=4, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='int32_value', full_name='protorpc.RepeatedMessage.int32_value', index=4,
      number=5, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='bool_value', full_name='protorpc.RepeatedMessage.bool_value', index=5,
      number=6, type=8, cpp_type=7, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='string_value', full_name='protorpc.RepeatedMessage.string_value', index=6,
      number=7, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='bytes_value', full_name='protorpc.RepeatedMessage.bytes_value', index=7,
      number=8, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='enum_value', full_name='protorpc.RepeatedMessage.enum_value', index=8,
      number=10, type=14, cpp_type=8, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _REPEATEDMESSAGE_SIMPLEENUM,
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=503,
  serialized_end=782,
)


_HASOPTIONALNESTEDMESSAGE = descriptor.Descriptor(
  name='HasOptionalNestedMessage',
  full_name='protorpc.HasOptionalNestedMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='nested', full_name='protorpc.HasOptionalNestedMessage.nested', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='repeated_nested', full_name='protorpc.HasOptionalNestedMessage.repeated_nested', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=784,
  serialized_end=905,
)

_HASNESTEDMESSAGE.fields_by_name['nested'].message_type = _NESTEDMESSAGE
_HASNESTEDMESSAGE.fields_by_name['repeated_nested'].message_type = _NESTEDMESSAGE
_OPTIONALMESSAGE.fields_by_name['enum_value'].enum_type = _OPTIONALMESSAGE_SIMPLEENUM
_OPTIONALMESSAGE_SIMPLEENUM.containing_type = _OPTIONALMESSAGE;
_REPEATEDMESSAGE.fields_by_name['enum_value'].enum_type = _REPEATEDMESSAGE_SIMPLEENUM
_REPEATEDMESSAGE_SIMPLEENUM.containing_type = _REPEATEDMESSAGE;
_HASOPTIONALNESTEDMESSAGE.fields_by_name['nested'].message_type = _OPTIONALMESSAGE
_HASOPTIONALNESTEDMESSAGE.fields_by_name['repeated_nested'].message_type = _OPTIONALMESSAGE
DESCRIPTOR.message_types_by_name['NestedMessage'] = _NESTEDMESSAGE
DESCRIPTOR.message_types_by_name['HasNestedMessage'] = _HASNESTEDMESSAGE
DESCRIPTOR.message_types_by_name['HasDefault'] = _HASDEFAULT
DESCRIPTOR.message_types_by_name['OptionalMessage'] = _OPTIONALMESSAGE
DESCRIPTOR.message_types_by_name['RepeatedMessage'] = _REPEATEDMESSAGE
DESCRIPTOR.message_types_by_name['HasOptionalNestedMessage'] = _HASOPTIONALNESTEDMESSAGE

class NestedMessage(six.with_metaclass(reflection.GeneratedProtocolMessageType, message.Message)):
  DESCRIPTOR = _NESTEDMESSAGE
  
  # @@protoc_insertion_point(class_scope:protorpc.NestedMessage)

class HasNestedMessage(six.with_metaclass(reflection.GeneratedProtocolMessageType, message.Message)):
  DESCRIPTOR = _HASNESTEDMESSAGE
  
  # @@protoc_insertion_point(class_scope:protorpc.HasNestedMessage)

class HasDefault(six.with_metaclass(reflection.GeneratedProtocolMessageType, message.Message)):
  DESCRIPTOR = _HASDEFAULT
  
  # @@protoc_insertion_point(class_scope:protorpc.HasDefault)

class OptionalMessage(six.with_metaclass(reflection.GeneratedProtocolMessageType, message.Message)):
  DESCRIPTOR = _OPTIONALMESSAGE
  
  # @@protoc_insertion_point(class_scope:protorpc.OptionalMessage)

class RepeatedMessage(six.with_metaclass(reflection.GeneratedProtocolMessageType, message.Message)):
  DESCRIPTOR = _REPEATEDMESSAGE
  
  # @@protoc_insertion_point(class_scope:protorpc.RepeatedMessage)

class HasOptionalNestedMessage(six.with_metaclass(reflection.GeneratedProtocolMessageType, message.Message)):
  DESCRIPTOR = _HASOPTIONALNESTEDMESSAGE
  
  # @@protoc_insertion_point(class_scope:protorpc.HasOptionalNestedMessage)

# @@protoc_insertion_point(module_scope)



#!/usr/bin/env python
#
# Copyright 2010 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""JSON support for message types.

Public classes:
  MessageJSONEncoder: JSON encoder for message objects.

Public functions:
  encode_message: Encodes a message in to a JSON string.
  decode_message: Merge from a JSON string in to a message.
"""
import six

__author__ = 'rafek@google.com (Rafe Kaplan)'

import base64
import binascii
import logging

from . import message_types
from . import messages
from . import util

__all__ = [
    'ALTERNATIVE_CONTENT_TYPES',
    'CONTENT_TYPE',
    'MessageJSONEncoder',
    'encode_message',
    'decode_message',
    'ProtoJson',
]


def _load_json_module():
  """Try to load a valid json module.

  There are more than one json modules that might be installed.  They are
  mostly compatible with one another but some versions may be different.
  This function attempts to load various json modules in a preferred order.
  It does a basic check to guess if a loaded version of json is compatible.

  Returns:
    Compatible json module.

  Raises:
    ImportError if there are no json modules or the loaded json module is
      not compatible with ProtoRPC.
  """
  first_import_error = None
  for module_name in ['json',
                      'simplejson']:
    try:
      module = __import__(module_name, {}, {}, 'json')
      if not hasattr(module, 'JSONEncoder'):
        message = ('json library "%s" is not compatible with ProtoRPC' %
                   module_name)
        logging.warning(message)
        raise ImportError(message)
      else:
        return module
    except ImportError as err:
      if not first_import_error:
        first_import_error = err

  logging.error('Must use valid json library (Python 2.6 json or simplejson)')
  raise first_import_error
json = _load_json_module()


# TODO: Rename this to MessageJsonEncoder.
class MessageJSONEncoder(json.JSONEncoder):
  """Message JSON encoder class.

  Extension of JSONEncoder that can build JSON from a message object.
  """

  def __init__(self, protojson_protocol=None, **kwargs):
    """Constructor.

    Args:
      protojson_protocol: ProtoJson instance.
    """
    super(MessageJSONEncoder, self).__init__(**kwargs)
    self.__protojson_protocol = protojson_protocol or ProtoJson.get_default()

  def default(self, value):
    """Return dictionary instance from a message object.

    Args:
    value: Value to get dictionary for.  If not encodable, will
      call superclasses default method.
    """
    if isinstance(value, messages.Enum):
      return str(value)

    if six.PY3 and isinstance(value, bytes):
      return value.decode('utf8')

    if isinstance(value, messages.Message):
      result = {}
      for field in value.all_fields():
        item = value.get_assigned_value(field.name)
        if item not in (None, [], ()):
          result[field.name] = self.__protojson_protocol.encode_field(
             field, item)
      # Handle unrecognized fields, so they're included when a message is
      # decoded then encoded.
      for unknown_key in value.all_unrecognized_fields():
        unrecognized_field, _ = value.get_unrecognized_field_info(unknown_key)
        result[unknown_key] = unrecognized_field
      return result
    else:
      return super(MessageJSONEncoder, self).default(value)


class ProtoJson(object):
  """ProtoRPC JSON implementation class.

  Implementation of JSON based protocol used for serializing and deserializing
  message objects.  Instances of remote.ProtocolConfig constructor or used with
  remote.Protocols.add_protocol.  See the remote.py module for more details.
  """

  CONTENT_TYPE = 'application/json'
  ALTERNATIVE_CONTENT_TYPES = [
      'application/x-javascript',
      'text/javascript',
      'text/x-javascript',
      'text/x-json',
      'text/json',
  ]

  def encode_field(self, field, value):
    """Encode a python field value to a JSON value.

    Args:
      field: A ProtoRPC field instance.
      value: A python value supported by field.

    Returns:
      A JSON serializable value appropriate for field.
    """
    if isinstance(field, messages.BytesField):
      if field.repeated:
        value = [base64.b64encode(byte) for byte in value]
      else:
        value = base64.b64encode(value)
    elif isinstance(field, message_types.DateTimeField):
      # DateTimeField stores its data as a RFC 3339 compliant string.
      if field.repeated:
        value = [i.isoformat() for i in value]
      else:
        value = value.isoformat()
    return value

  def encode_message(self, message):
    """Encode Message instance to JSON string.

    Args:
      Message instance to encode in to JSON string.

    Returns:
      String encoding of Message instance in protocol JSON format.

    Raises:
      messages.ValidationError if message is not initialized.
    """
    message.check_initialized()

    return json.dumps(message, cls=MessageJSONEncoder, protojson_protocol=self)

  def decode_message(self, message_type, encoded_message):
    """Merge JSON structure to Message instance.

    Args:
      message_type: Message to decode data to.
      encoded_message: JSON encoded version of message.

    Returns:
      Decoded instance of message_type.

    Raises:
      ValueError: If encoded_message is not valid JSON.
      messages.ValidationError if merged message is not initialized.
    """
    dictionary = json.loads(encoded_message) if encoded_message.strip() else {}
    message = self.__decode_dictionary(message_type, dictionary)
    message.check_initialized()
    return message

  def __find_variant(self, value):
    """Find the messages.Variant type that describes this value.

    Args:
      value: The value whose variant type is being determined.

    Returns:
      The messages.Variant value that best describes value's type, or None if
      it's a type we don't know how to handle.
    """
    if isinstance(value, bool):
      return messages.Variant.BOOL
    elif isinstance(value, six.integer_types):
      return messages.Variant.INT64
    elif isinstance(value, float):
      return messages.Variant.DOUBLE
    elif isinstance(value, six.string_types):
      return messages.Variant.STRING
    elif isinstance(value, (list, tuple)):
      # Find the most specific variant that covers all elements.
      variant_priority = [None, messages.Variant.INT64, messages.Variant.DOUBLE,
                          messages.Variant.STRING]
      chosen_priority = 0
      for v in value:
        variant = self.__find_variant(v)
        try:
          priority = variant_priority.index(variant)
        except IndexError:
          priority = -1
        if priority > chosen_priority:
          chosen_priority = priority
      return variant_priority[chosen_priority]
    # Unrecognized type.
    return None

  def __decode_dictionary(self, message_type, dictionary):
    """Merge dictionary in to message.

    Args:
      message: Message to merge dictionary in to.
      dictionary: Dictionary to extract information from.  Dictionary
        is as parsed from JSON.  Nested objects will also be dictionaries.
    """
    message = message_type()
    for key, value in six.iteritems(dictionary):
      if value is None:
        try:
          message.reset(key)
        except AttributeError:
          pass  # This is an unrecognized field, skip it.
        continue

      try:
        field = message.field_by_name(key)
      except KeyError:
        # Save unknown values.
        variant = self.__find_variant(value)
        if variant:
          if key.isdigit():
            key = int(key)
          message.set_unrecognized_field(key, value, variant)
        else:
          logging.warning('No variant found for unrecognized field: %s', key)
        continue

      # Normalize values in to a list.
      if isinstance(value, list):
        if not value:
          continue
      else:
        value = [value]

      valid_value = []
      for item in value:
        valid_value.append(self.decode_field(field, item))

      if field.repeated:
        existing_value = getattr(message, field.name)
        setattr(message, field.name, valid_value)
      else:
        setattr(message, field.name, valid_value[-1])
    return message

  def decode_field(self, field, value):
    """Decode a JSON value to a python value.

    Args:
      field: A ProtoRPC field instance.
      value: A serialized JSON value.

    Return:
      A Python value compatible with field.
    """
    if isinstance(field, messages.EnumField):
      try:
        return field.type(value)
      except TypeError:
        raise messages.DecodeError('Invalid enum value "%s"' % (value or ''))

    elif isinstance(field, messages.BytesField):
      try:
        return base64.b64decode(value)
      except (binascii.Error, TypeError) as err:
        raise messages.DecodeError('Base64 decoding error: %s' % err)

    elif isinstance(field, message_types.DateTimeField):
      try:
        return util.decode_datetime(value)
      except ValueError as err:
        raise messages.DecodeError(err)

    elif (isinstance(field, messages.MessageField) and
          issubclass(field.type, messages.Message)):
      return self.__decode_dictionary(field.type, value)

    elif (isinstance(field, messages.FloatField) and
          isinstance(value, (six.integer_types, six.string_types))):
      try:
        return float(value)
      except:
        pass

    elif (isinstance(field, messages.IntegerField) and
          isinstance(value, six.string_types)):
      try:
        return int(value)
      except:
        pass

    return value

  @staticmethod
  def get_default():
    """Get default instanceof ProtoJson."""
    try:
      return ProtoJson.__default
    except AttributeError:
      ProtoJson.__default = ProtoJson()
      return ProtoJson.__default

  @staticmethod
  def set_default(protocol):
    """Set the default instance of ProtoJson.

    Args:
      protocol: A ProtoJson instance.
    """
    if not isinstance(protocol, ProtoJson):
      raise TypeError('Expected protocol of type ProtoJson')
    ProtoJson.__default = protocol

CONTENT_TYPE = ProtoJson.CONTENT_TYPE

ALTERNATIVE_CONTENT_TYPES = ProtoJson.ALTERNATIVE_CONTENT_TYPES

encode_message = ProtoJson.get_default().encode_message

decode_message = ProtoJson.get_default().decode_message



#!/usr/bin/env python
#
# Copyright 2010 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""Remote service library.

This module contains classes that are useful for building remote services that
conform to a standard request and response model.  To conform to this model
a service must be like the following class:

  # Each service instance only handles a single request and is then discarded.
  # Make these objects light weight.
  class Service(object):

    # It must be possible to construct service objects without any parameters.
    # If your constructor needs extra information you should provide a
    # no-argument factory function to create service instances.
    def __init__(self):
      ...

    # Each remote method must use the 'method' decorator, passing the request
    # and response message types.  The remote method itself must take a single
    # parameter which is an instance of RequestMessage and return an instance
    # of ResponseMessage.
    @method(RequestMessage, ResponseMessage)
    def remote_method(self, request):
      # Return an instance of ResponseMessage.

    # A service object may optionally implement an 'initialize_request_state'
    # method that takes as a parameter a single instance of a RequestState.  If
    # a service does not implement this method it will not receive the request
    # state.
    def initialize_request_state(self, state):
      ...

The 'Service' class is provided as a convenient base class that provides the
above functionality.  It implements all required and optional methods for a
service.  It also has convenience methods for creating factory functions that
can pass persistent global state to a new service instance.

The 'method' decorator is used to declare which methods of a class are
meant to service RPCs.  While this decorator is not responsible for handling
actual remote method invocations, such as handling sockets, handling various
RPC protocols and checking messages for correctness, it does attach information
to methods that responsible classes can examine and ensure the correctness
of the RPC.

When the method decorator is used on a method, the wrapper method will have a
'remote' property associated with it.  The 'remote' property contains the
request_type and response_type expected by the methods implementation.

On its own, the method decorator does not provide any support for subclassing
remote methods.  In order to extend a service, one would need to redecorate
the sub-classes methods.  For example:

  class MyService(Service):

    @method(DoSomethingRequest, DoSomethingResponse)
    def do_stuff(self, request):
      ... implement do_stuff ...

  class MyBetterService(MyService):

    @method(DoSomethingRequest, DoSomethingResponse)
    def do_stuff(self, request):
      response = super(MyBetterService, self).do_stuff.remote.method(request)
      ... do stuff with response ...
      return response

A Service subclass also has a Stub class that can be used with a transport for
making RPCs.  When a stub is created, it is capable of doing both synchronous
and asynchronous RPCs if the underlying transport supports it.  To make a stub
using an HTTP transport do:

  my_service = MyService.Stub(HttpTransport('<my service URL>'))

For synchronous calls, just call the expected methods on the service stub:

  request = DoSomethingRequest()
  ...
  response = my_service.do_something(request)

Each stub instance has an async object that can be used for initiating
asynchronous RPCs if the underlying protocol transport supports it.  To
make an asynchronous call, do:

  rpc = my_service.proto_async.do_something(request)
  response = rpc.get_response()
"""

from __future__ import with_statement
import six

__author__ = 'rafek@google.com (Rafe Kaplan)'

import functools
import logging
import sys
import threading
from wsgiref import headers as wsgi_headers

from . import message_types
from . import messages
from . import protobuf
from . import protojson
from . import util


__all__ = [
    'ApplicationError',
    'MethodNotFoundError',
    'NetworkError',
    'RequestError',
    'RpcError',
    'ServerError',
    'ServiceConfigurationError',
    'ServiceDefinitionError',

    'HttpRequestState',
    'ProtocolConfig',
    'Protocols',
    'RequestState',
    'RpcState',
    'RpcStatus',
    'Service',
    'StubBase',
    'check_rpc_status',
    'get_remote_method_info',
    'is_error_status',
    'method',
    'remote',
]


class ServiceDefinitionError(messages.Error):
  """Raised when a service is improperly defined."""


class ServiceConfigurationError(messages.Error):
  """Raised when a service is incorrectly configured."""


# TODO: Use error_name to map to specific exception message types.
class RpcStatus(messages.Message):
  """Status of on-going or complete RPC.

  Fields:
    state: State of RPC.
    error_name: Error name set by application.  Only set when
      status is APPLICATION_ERROR.  For use by application to transmit
      specific reason for error.
    error_message: Error message associated with status.
  """

  class State(messages.Enum):
    """Enumeration of possible RPC states.

    Values:
      OK: Completed successfully.
      RUNNING: Still running, not complete.
      REQUEST_ERROR: Request was malformed or incomplete.
      SERVER_ERROR: Server experienced an unexpected error.
      NETWORK_ERROR: An error occured on the network.
      APPLICATION_ERROR: The application is indicating an error.
        When in this state, RPC should also set application_error.
    """
    OK = 0
    RUNNING = 1

    REQUEST_ERROR = 2
    SERVER_ERROR = 3
    NETWORK_ERROR = 4
    APPLICATION_ERROR = 5
    METHOD_NOT_FOUND_ERROR = 6

  state = messages.EnumField(State, 1, required=True)
  error_message = messages.StringField(2)
  error_name = messages.StringField(3)


RpcState = RpcStatus.State


class RpcError(messages.Error):
  """Base class for RPC errors.

  Each sub-class of RpcError is associated with an error value from RpcState
  and has an attribute STATE that refers to that value.
  """

  def __init__(self, message, cause=None):
    super(RpcError, self).__init__(message)
    self.cause = cause

  @classmethod
  def from_state(cls, state):
    """Get error class from RpcState.

    Args:
      state: RpcState value.  Can be enum value itself, string or int.

    Returns:
      Exception class mapped to value if state is an error.  Returns None
      if state is OK or RUNNING.
    """
    return _RPC_STATE_TO_ERROR.get(RpcState(state))


class RequestError(RpcError):
  """Raised when wrong request objects received during method invocation."""

  STATE = RpcState.REQUEST_ERROR


class MethodNotFoundError(RequestError):
  """Raised when unknown method requested by RPC."""

  STATE = RpcState.METHOD_NOT_FOUND_ERROR


class NetworkError(RpcError):
  """Raised when network error occurs during RPC."""

  STATE = RpcState.NETWORK_ERROR


class ServerError(RpcError):
  """Unexpected error occured on server."""

  STATE = RpcState.SERVER_ERROR


class ApplicationError(RpcError):
  """Raised for application specific errors.

  Attributes:
    error_name: Application specific error name for exception.
  """

  STATE = RpcState.APPLICATION_ERROR

  def __init__(self, message, error_name=None):
    """Constructor.

    Args:
      message: Application specific error message.
      error_name: Application specific error name.  Must be None, string
      or unicode string.
    """
    super(ApplicationError, self).__init__(message)
    self.error_name = error_name

  def __str__(self):
    return self.args[0] or ''

  def __repr__(self):
    if self.error_name is None:
      error_format = ''
    else:
      error_format = ', %r' % self.error_name
    return '%s(%r%s)' % (type(self).__name__, self.args[0], error_format)


_RPC_STATE_TO_ERROR = {
  RpcState.REQUEST_ERROR: RequestError,
  RpcState.NETWORK_ERROR: NetworkError,
  RpcState.SERVER_ERROR: ServerError,
  RpcState.APPLICATION_ERROR: ApplicationError,
  RpcState.METHOD_NOT_FOUND_ERROR: MethodNotFoundError,
}

class _RemoteMethodInfo(object):
  """Object for encapsulating remote method information.

  An instance of this method is associated with the 'remote' attribute
  of the methods 'invoke_remote_method' instance.

  Instances of this class are created by the remote decorator and should not
  be created directly.
  """

  def __init__(self,
               method,
               request_type,
               response_type):
    """Constructor.

    Args:
      method: The method which implements the remote method.  This is a
        function that will act as an instance method of a class definition
        that is decorated by '@method'.  It must always take 'self' as its
        first parameter.
      request_type: Expected request type for the remote method.
      response_type: Expected response type for the remote method.
    """
    self.__method = method
    self.__request_type = request_type
    self.__response_type = response_type

  @property
  def method(self):
    """Original undecorated method."""
    return self.__method

  @property
  def request_type(self):
    """Expected request type for remote method."""
    if isinstance(self.__request_type, six.string_types):
      self.__request_type = messages.find_definition(
        self.__request_type,
        relative_to=sys.modules[self.__method.__module__])
    return self.__request_type

  @property
  def response_type(self):
    """Expected response type for remote method."""
    if isinstance(self.__response_type, six.string_types):
      self.__response_type = messages.find_definition(
        self.__response_type,
        relative_to=sys.modules[self.__method.__module__])
    return self.__response_type


def method(request_type=message_types.VoidMessage,
           response_type=message_types.VoidMessage):
  """Method decorator for creating remote methods.

  Args:
    request_type: Message type of expected request.
    response_type: Message type of expected response.

  Returns:
    'remote_method_wrapper' function.

  Raises:
    TypeError: if the request_type or response_type parameters are not
      proper subclasses of messages.Message.
  """
  if (not isinstance(request_type, six.string_types) and
      (not isinstance(request_type, type) or
       not issubclass(request_type, messages.Message) or
       request_type is messages.Message)):
    raise TypeError(
        'Must provide message class for request-type.  Found %s',
        request_type)

  if (not isinstance(response_type, six.string_types) and
      (not isinstance(response_type, type) or
       not issubclass(response_type, messages.Message) or
       response_type is messages.Message)):
    raise TypeError(
        'Must provide message class for response-type.  Found %s',
        response_type)

  def remote_method_wrapper(method):
    """Decorator used to wrap method.

    Args:
      method: Original method being wrapped.

    Returns:
      'invoke_remote_method' function responsible for actual invocation.
      This invocation function instance is assigned an attribute 'remote'
      which contains information about the remote method:
        request_type: Expected request type for remote method.
        response_type: Response type returned from remote method.

    Raises:
      TypeError: If request_type or response_type is not a subclass of Message
        or is the Message class itself.
    """

    @functools.wraps(method)
    def invoke_remote_method(service_instance, request):
      """Function used to replace original method.

      Invoke wrapped remote method.  Checks to ensure that request and
      response objects are the correct types.

      Does not check whether messages are initialized.

      Args:
        service_instance: The service object whose method is being invoked.
          This is passed to 'self' during the invocation of the original
          method.
        request: Request message.

      Returns:
        Results of calling wrapped remote method.

      Raises:
        RequestError: Request object is not of the correct type.
        ServerError: Response object is not of the correct type.
      """
      if not isinstance(request, remote_method_info.request_type):
        raise RequestError('Method %s.%s expected request type %s, '
                           'received %s' %
                           (type(service_instance).__name__,
                            method.__name__,
                            remote_method_info.request_type,
                            type(request)))
      response = method(service_instance, request)
      if not isinstance(response, remote_method_info.response_type):
        raise ServerError('Method %s.%s expected response type %s, '
                          'sent %s' %
                          (type(service_instance).__name__,
                           method.__name__,
                           remote_method_info.response_type,
                           type(response)))
      return response

    remote_method_info = _RemoteMethodInfo(method,
                                           request_type,
                                           response_type)

    invoke_remote_method.remote = remote_method_info
    return invoke_remote_method

  return remote_method_wrapper


def remote(request_type, response_type):
  """Temporary backward compatibility alias for method."""
  logging.warning('The remote decorator has been renamed method.  It will be '
                  'removed in very soon from future versions of ProtoRPC.')
  return method(request_type, response_type)


def get_remote_method_info(method):
  """Get remote method info object from remote method.

  Returns:
    Remote method info object if method is a remote method, else None.
  """
  if not callable(method):
    return None

  try:
    method_info = method.remote
  except AttributeError:
    return None

  if not isinstance(method_info, _RemoteMethodInfo):
    return None

  return method_info


class StubBase(object):
  """Base class for client side service stubs.

  The remote method stubs are created by the _ServiceClass meta-class
  when a Service class is first created.  The resulting stub will
  extend both this class and the service class it handles communications for.

  Assume that there is a service:

    class NewContactRequest(messages.Message):

      name = messages.StringField(1, required=True)
      phone = messages.StringField(2)
      email = messages.StringField(3)

    class NewContactResponse(message.Message):

      contact_id = messages.StringField(1)

    class AccountService(remote.Service):

      @remote.method(NewContactRequest, NewContactResponse):
      def new_contact(self, request):
        ... implementation ...

  A stub of this service can be called in two ways.  The first is to pass in a
  correctly initialized NewContactRequest message:

    request = NewContactRequest()
    request.name = 'Bob Somebody'
    request.phone = '+1 415 555 1234'

    response = account_service_stub.new_contact(request)

  The second way is to pass in keyword parameters that correspond with the root
  request message type:

      account_service_stub.new_contact(name='Bob Somebody',
                                       phone='+1 415 555 1234')

  The second form will create a request message of the appropriate type.
  """

  def __init__(self, transport):
    """Constructor.

    Args:
      transport: Underlying transport to communicate with remote service.
    """
    self.__transport = transport

  @property
  def transport(self):
    """Transport used to communicate with remote service."""
    return self.__transport


class _ServiceClass(type):
  """Meta-class for service class."""

  def __new_async_method(cls, remote):
    """Create asynchronous method for Async handler.

    Args:
      remote: RemoteInfo to create method for.
    """
    def async_method(self, *args, **kwargs):
      """Asynchronous remote method.

      Args:
        self: Instance of StubBase.Async subclass.

        Stub methods either take a single positional argument when a full
        request message is passed in, or keyword arguments, but not both.

        See docstring for StubBase for more information on how to use remote
        stub methods.

      Returns:
        Rpc instance used to represent asynchronous RPC.
      """
      if args and kwargs:
        raise TypeError('May not provide both args and kwargs')

      if not args:
        # Construct request object from arguments.
        request = remote.request_type()
        for name, value in six.iteritems(kwargs):
          setattr(request, name, value)
      else:
        # First argument is request object.
        request = args[0]

      return self.transport.send_rpc(remote, request)

    async_method.__name__ = remote.method.__name__
    async_method = util.positional(2)(async_method)
    async_method.remote = remote
    return async_method

  def __new_sync_method(cls, async_method):
    """Create synchronous method for stub.

    Args:
      async_method: asynchronous method to delegate calls to.
    """
    def sync_method(self, *args, **kwargs):
      """Synchronous remote method.

      Args:
        self: Instance of StubBase.Async subclass.
        args: Tuple (request,):
          request: Request object.
        kwargs: Field values for request.  Must be empty if request object
          is provided.

      Returns:
        Response message from synchronized RPC.
      """
      return async_method(self.proto_async, *args, **kwargs).response
    sync_method.__name__ = async_method.__name__
    sync_method.remote = async_method.remote
    return sync_method

  def __create_async_methods(cls, remote_methods):
    """Construct a dictionary of asynchronous methods based on remote methods.

    Args:
      remote_methods: Dictionary of methods with associated RemoteInfo objects.

    Returns:
      Dictionary of asynchronous methods with assocaited RemoteInfo objects.
      Results added to AsyncStub subclass.
    """
    async_methods = {}
    for method_name, method in remote_methods.items():
      async_methods[method_name] = cls.__new_async_method(method.remote)
    return async_methods

  def __create_sync_methods(cls, async_methods):
    """Construct a dictionary of synchronous methods based on remote methods.

    Args:
      async_methods: Dictionary of async methods to delegate calls to.

    Returns:
      Dictionary of synchronous methods with assocaited RemoteInfo objects.
      Results added to Stub subclass.
    """
    sync_methods = {}
    for method_name, async_method in async_methods.items():
      sync_methods[method_name] = cls.__new_sync_method(async_method)
    return sync_methods

  def __new__(cls, name, bases, dct):
    """Instantiate new service class instance."""
    if StubBase not in bases:
      # Collect existing remote methods.
      base_methods = {}
      for base in bases:
        try:
          remote_methods = base.__remote_methods
        except AttributeError:
          pass
        else:
          base_methods.update(remote_methods)

      # Set this class private attribute so that base_methods do not have
      # to be recacluated in __init__.
      dct['_ServiceClass__base_methods'] = base_methods

      for attribute, value in dct.items():
        base_method = base_methods.get(attribute, None)
        if base_method:
          if not callable(value):
            raise ServiceDefinitionError(
              'Must override %s in %s with a method.' % (
                attribute, name))

          if get_remote_method_info(value):
            raise ServiceDefinitionError(
              'Do not use method decorator when overloading remote method %s '
              'on service %s.' %
              (attribute, name))

          base_remote_method_info = get_remote_method_info(base_method)
          remote_decorator = method(
            base_remote_method_info.request_type,
            base_remote_method_info.response_type)
          new_remote_method = remote_decorator(value)
          dct[attribute] = new_remote_method

    return type.__new__(cls, name, bases, dct)

  def __init__(cls, name, bases, dct):
    """Create uninitialized state on new class."""
    type.__init__(cls, name, bases, dct)

    # Only service implementation classes should have remote methods and stub
    # sub classes created.  Stub implementations have their own methods passed
    # in to the type constructor.
    if StubBase not in bases:
      # Create list of remote methods.
      cls.__remote_methods = dict(cls.__base_methods)

      for attribute, value in dct.items():
        value = getattr(cls, attribute)
        remote_method_info = get_remote_method_info(value)
        if remote_method_info:
          cls.__remote_methods[attribute] = value

      # Build asynchronous stub class.
      stub_attributes = {'Service': cls}
      async_methods = cls.__create_async_methods(cls.__remote_methods)
      stub_attributes.update(async_methods)
      async_class = type('AsyncStub', (StubBase, cls), stub_attributes)
      cls.AsyncStub = async_class

      # Constructor for synchronous stub class.
      def __init__(self, transport):
        """Constructor.

        Args:
          transport: Underlying transport to communicate with remote service.
        """
        super(cls.Stub, self).__init__(transport)
        self.proto_async = cls.AsyncStub(transport)

      # Build synchronous stub class.
      stub_attributes = {'Service': cls,
                         '__init__': __init__}
      stub_attributes.update(cls.__create_sync_methods(async_methods))

      cls.Stub = type('Stub', (StubBase, cls), stub_attributes)

  @staticmethod
  def all_remote_methods(cls):
    """Get all remote methods of service.

    Returns:
      Dict from method name to unbound method.
    """
    return dict(cls.__remote_methods)


class RequestState(object):
  """Request state information.

  Properties:
    remote_host: Remote host name where request originated.
    remote_address: IP address where request originated.
    server_host: Host of server within which service resides.
    server_port: Post which service has recevied request from.
  """

  @util.positional(1)
  def __init__(self,
               remote_host=None,
               remote_address=None,
               server_host=None,
               server_port=None):
    """Constructor.

    Args:
      remote_host: Assigned to property.
      remote_address: Assigned to property.
      server_host: Assigned to property.
      server_port: Assigned to property.
    """
    self.__remote_host = remote_host
    self.__remote_address = remote_address
    self.__server_host = server_host
    self.__server_port = server_port

  @property
  def remote_host(self):
    return self.__remote_host

  @property
  def remote_address(self):
    return self.__remote_address

  @property
  def server_host(self):
    return self.__server_host

  @property
  def server_port(self):
    return self.__server_port

  def _repr_items(self):
    for name in ['remote_host',
                 'remote_address',
                 'server_host',
                 'server_port']:
      yield name, getattr(self, name)

  def __repr__(self):
    """String representation of state."""
    state = [self.__class__.__name__]
    for name, value in self._repr_items():
      if value:
        state.append('%s=%r' % (name, value))

    return '<%s>' % (' '.join(state),)


class HttpRequestState(RequestState):
  """HTTP request state information.

  NOTE: Does not attempt to represent certain types of information from the
  request such as the query string as query strings are not permitted in
  ProtoRPC URLs unless required by the underlying message format.

  Properties:
    headers: wsgiref.headers.Headers instance of HTTP request headers.
    http_method: HTTP method as a string.
    service_path: Path on HTTP service where service is mounted.  This path
      will not include the remote method name.
  """

  @util.positional(1)
  def __init__(self,
               http_method=None,
               service_path=None,
               headers=None,
               **kwargs):
    """Constructor.

    Args:
      Same as RequestState, including:
        http_method: Assigned to property.
        service_path: Assigned to property.
        headers: HTTP request headers.  If instance of Headers, assigned to
          property without copying.  If dict, will convert to name value pairs
          for use with Headers constructor.  Otherwise, passed as parameters to
          Headers constructor.
    """
    super(HttpRequestState, self).__init__(**kwargs)

    self.__http_method = http_method
    self.__service_path = service_path

    # Initialize headers.
    if isinstance(headers, dict):
      header_list = []
      for key, value in sorted(headers.items()):
        if not isinstance(value, list):
          value = [value]
        for item in value:
          header_list.append((key, item))
        headers = header_list
    self.__headers = wsgi_headers.Headers(headers or [])

  @property
  def http_method(self):
    return self.__http_method

  @property
  def service_path(self):
    return self.__service_path

  @property
  def headers(self):
    return self.__headers

  def _repr_items(self):
    for item in super(HttpRequestState, self)._repr_items():
      yield item

    for name in ['http_method', 'service_path']:
      yield name, getattr(self, name)

    yield 'headers', list(self.headers.items())


class Service(six.with_metaclass(_ServiceClass, object)):
  """Service base class.

  Base class used for defining remote services.  Contains reflection functions,
  useful helpers and built-in remote methods.

  Services are expected to be constructed via either a constructor or factory
  which takes no parameters.  However, it might be required that some state or
  configuration is passed in to a service across multiple requests.

  To do this, define parameters to the constructor of the service and use
  the 'new_factory' class method to build a constructor that will transmit
  parameters to the constructor.  For example:

    class MyService(Service):

      def __init__(self, configuration, state):
        self.configuration = configuration
        self.state = state

    configuration = MyServiceConfiguration()
    global_state = MyServiceState()

    my_service_factory = MyService.new_factory(configuration,
                                               state=global_state)

  The contract with any service handler is that a new service object is created
  to handle each user request, and that the construction does not take any
  parameters.  The factory satisfies this condition:

    new_instance = my_service_factory()
    assert new_instance.state is global_state

  Attributes:
    request_state: RequestState set via initialize_request_state.
  """

  __request_state = None

  @classmethod
  def all_remote_methods(cls):
    """Get all remote methods for service class.

    Built-in methods do not appear in the dictionary of remote methods.

    Returns:
      Dictionary mapping method name to remote method.
    """
    return _ServiceClass.all_remote_methods(cls)

  @classmethod
  def new_factory(cls, *args, **kwargs):
    """Create factory for service.

    Useful for passing configuration or state objects to the service.  Accepts
    arbitrary parameters and keywords, however, underlying service must accept
    also accept not other parameters in its constructor.

    Args:
      args: Args to pass to service constructor.
      kwargs: Keyword arguments to pass to service constructor.

    Returns:
      Factory function that will create a new instance and forward args and
      keywords to the constructor.
    """

    def service_factory():
      return cls(*args, **kwargs)

    # Update docstring so that it is easier to debug.
    full_class_name = '%s.%s' % (cls.__module__, cls.__name__)
    service_factory.__doc__ = (
        'Creates new instances of service %s.\n\n'
        'Returns:\n'
        '  New instance of %s.'
        % (cls.__name__, full_class_name))

    # Update name so that it is easier to debug the factory function.
    service_factory.__name__ = '%s_service_factory' % cls.__name__

    service_factory.service_class = cls

    return service_factory

  def initialize_request_state(self, request_state):
    """Save request state for use in remote method.

    Args:
      request_state: RequestState instance.
    """
    self.__request_state = request_state

  @classmethod
  def definition_name(cls):
    """Get definition name for Service class.

    Package name is determined by the global 'package' attribute in the
    module that contains the Service definition.  If no 'package' attribute
    is available, uses module name.  If no module is found, just uses class
    name as name.

    Returns:
      Fully qualified service name.
    """
    try:
      return cls.__definition_name
    except AttributeError:
      outer_definition_name = cls.outer_definition_name()
      if outer_definition_name is None:
        cls.__definition_name = cls.__name__
      else:
        cls.__definition_name = '%s.%s' % (outer_definition_name, cls.__name__)

      return cls.__definition_name

  @classmethod
  def outer_definition_name(cls):
    """Get outer definition name.

    Returns:
      Package for service.  Services are never nested inside other definitions.
    """
    return cls.definition_package()

  @classmethod
  def definition_package(cls):
    """Get package for service.

    Returns:
      Package name for service.
    """
    try:
      return cls.__definition_package
    except AttributeError:
      cls.__definition_package = util.get_package_for_module(cls.__module__)

    return cls.__definition_package

  @property
  def request_state(self):
    """Request state associated with this Service instance."""
    return self.__request_state


def is_error_status(status):
  """Function that determines whether the RPC status is an error.

  Args:
    status: Initialized RpcStatus message to check for errors.
  """
  status.check_initialized()
  return RpcError.from_state(status.state) is not None


def check_rpc_status(status):
  """Function converts an error status to a raised exception.

  Args:
    status: Initialized RpcStatus message to check for errors.

  Raises:
    RpcError according to state set on status, if it is an error state.
  """
  status.check_initialized()
  error_class = RpcError.from_state(status.state)
  if error_class is not None:
    if error_class is ApplicationError:
      raise error_class(status.error_message, status.error_name)
    else:
      raise error_class(status.error_message)


class ProtocolConfig(object):
  """Configuration for single protocol mapping.

  A read-only protocol configuration provides a given protocol implementation
  with a name and a set of content-types that it recognizes.

  Properties:
    protocol: The protocol implementation for configuration (usually a module,
      for example, protojson, protobuf, etc.).  This is an object that has the
      following attributes:
        CONTENT_TYPE: Used as the default content-type if default_content_type
          is not set.
        ALTERNATIVE_CONTENT_TYPES (optional): A list of alternative
          content-types to the default that indicate the same protocol.
        encode_message: Function that matches the signature of
          ProtocolConfig.encode_message.  Used for encoding a ProtoRPC message.
        decode_message: Function that matches the signature of
          ProtocolConfig.decode_message.  Used for decoding a ProtoRPC message.
    name: Name of protocol configuration.
    default_content_type: The default content type for the protocol.  Overrides
      CONTENT_TYPE defined on protocol.
    alternative_content_types: A list of alternative content-types supported
      by the protocol.  Must not contain the default content-type, nor
      duplicates.  Overrides ALTERNATIVE_CONTENT_TYPE defined on protocol.
    content_types: A list of all content-types supported by configuration.
      Combination of default content-type and alternatives.
  """

  def __init__(self,
               protocol,
               name,
               default_content_type=None,
               alternative_content_types=None):
    """Constructor.

    Args:
      protocol: The protocol implementation for configuration.
      name: The name of the protocol configuration.
      default_content_type: The default content-type for protocol.  If none
        provided it will check protocol.CONTENT_TYPE.
      alternative_content_types:  A list of content-types.  If none provided,
        it will check protocol.ALTERNATIVE_CONTENT_TYPES.  If that attribute
        does not exist, will be an empty tuple.

    Raises:
      ServiceConfigurationError if there are any duplicate content-types.
    """
    self.__protocol = protocol
    self.__name = name
    self.__default_content_type = (default_content_type or
                                   protocol.CONTENT_TYPE).lower()
    if alternative_content_types is None:
      alternative_content_types = getattr(protocol,
                                          'ALTERNATIVE_CONTENT_TYPES',
                                          ())
    self.__alternative_content_types = tuple(
      content_type.lower() for content_type in alternative_content_types)
    self.__content_types = (
      (self.__default_content_type,) + self.__alternative_content_types)

    # Detect duplicate content types in definition.
    previous_type = None
    for content_type in sorted(self.content_types):
      if content_type == previous_type:
        raise ServiceConfigurationError(
          'Duplicate content-type %s' % content_type)
      previous_type = content_type

  @property
  def protocol(self):
    return self.__protocol

  @property
  def name(self):
    return self.__name

  @property
  def default_content_type(self):
    return self.__default_content_type

  @property
  def alternate_content_types(self):
    return self.__alternative_content_types

  @property
  def content_types(self):
    return self.__content_types

  def encode_message(self, message):
    """Encode message.

    Args:
      message: Message instance to encode.

    Returns:
      String encoding of Message instance encoded in protocol's format.
    """
    return self.__protocol.encode_message(message)

  def decode_message(self, message_type, encoded_message):
    """Decode buffer to Message instance.

    Args:
      message_type: Message type to decode data to.
      encoded_message: Encoded version of message as string.

    Returns:
      Decoded instance of message_type.
    """
    return self.__protocol.decode_message(message_type, encoded_message)


class Protocols(object):
  """Collection of protocol configurations.

  Used to describe a complete set of content-type mappings for multiple
  protocol configurations.

  Properties:
    names: Sorted list of the names of registered protocols.
    content_types: Sorted list of supported content-types.
  """

  __default_protocols = None
  __lock = threading.Lock()

  def __init__(self):
    """Constructor."""
    self.__by_name = {}
    self.__by_content_type = {}

  def add_protocol_config(self, config):
    """Add a protocol configuration to protocol mapping.

    Args:
      config: A ProtocolConfig.

    Raises:
      ServiceConfigurationError if protocol.name is already registered
        or any of it's content-types are already registered.
    """
    if config.name in self.__by_name:
      raise ServiceConfigurationError(
        'Protocol name %r is already in use' % config.name)
    for content_type in config.content_types:
      if content_type in self.__by_content_type:
        raise ServiceConfigurationError(
          'Content type %r is already in use' % content_type)

    self.__by_name[config.name] = config
    self.__by_content_type.update((t, config) for t in config.content_types)

  def add_protocol(self, *args, **kwargs):
    """Add a protocol configuration from basic parameters.

    Simple helper method that creates and registeres a ProtocolConfig instance.
    """
    self.add_protocol_config(ProtocolConfig(*args, **kwargs))

  @property
  def names(self):
    return tuple(sorted(self.__by_name))

  @property
  def content_types(self):
    return tuple(sorted(self.__by_content_type))

  def lookup_by_name(self, name):
    """Look up a ProtocolConfig by name.

    Args:
      name: Name of protocol to look for.

    Returns:
      ProtocolConfig associated with name.

    Raises:
      KeyError if there is no protocol for name.
    """
    return self.__by_name[name.lower()]

  def lookup_by_content_type(self, content_type):
    """Look up a ProtocolConfig by content-type.

    Args:
      content_type: Content-type to find protocol configuration for.

    Returns:
      ProtocolConfig associated with content-type.

    Raises:
      KeyError if there is no protocol for content-type.
    """
    return self.__by_content_type[content_type.lower()]

  @classmethod
  def new_default(cls):
    """Create default protocols configuration.

    Returns:
      New Protocols instance configured for protobuf and protorpc.
    """
    protocols = cls()
    protocols.add_protocol(protobuf, 'protobuf')
    protocols.add_protocol(protojson.ProtoJson.get_default(), 'protojson')
    return protocols

  @classmethod
  def get_default(cls):
    """Get the global default Protocols instance.

    Returns:
      Current global default Protocols instance.
    """
    default_protocols = cls.__default_protocols
    if default_protocols is None:
      with cls.__lock:
        default_protocols = cls.__default_protocols
        if default_protocols is None:
          default_protocols = cls.new_default()
          cls.__default_protocols = default_protocols
    return default_protocols

  @classmethod
  def set_default(cls, protocols):
    """Set the global default Protocols instance.

    Args:
      protocols: A Protocols instance.

    Raises:
      TypeError: If protocols is not an instance of Protocols.
    """
    if not isinstance(protocols, Protocols):
      raise TypeError(
        'Expected value of type "Protocols", found %r' % protocols)
    with cls.__lock:
      cls.__default_protocols = protocols



#!/usr/bin/env python
#
# Copyright 2010 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from __future__ import with_statement

__author__ = 'rafek@google.com (Rafe Kaplan)'

import logging

from . import descriptor
from . import generate
from . import messages
from . import util


__all__ = ['format_proto_file']


@util.positional(2)
def format_proto_file(file_descriptor, output, indent_space=2):
  out = generate.IndentWriter(output, indent_space=indent_space)

  if file_descriptor.package:
    out << 'package %s;' % file_descriptor.package

  def write_enums(enum_descriptors):
    """Write nested and non-nested Enum types.

    Args:
      enum_descriptors: List of EnumDescriptor objects from which to generate
        enums.
    """
    # Write enums.
    for enum in enum_descriptors or []:
      out << ''
      out << ''
      out << 'enum %s {' % enum.name
      out << ''

      with out.indent():
        if enum.values:
          for enum_value in enum.values:
            out << '%s = %s;' % (enum_value.name, enum_value.number)

      out << '}'

  write_enums(file_descriptor.enum_types)

  def write_fields(field_descriptors):
    """Write fields for Message types.

    Args:
      field_descriptors: List of FieldDescriptor objects from which to generate
        fields.
    """
    for field in field_descriptors or []:
      default_format = ''
      if field.default_value is not None:
        if field.label == descriptor.FieldDescriptor.Label.REPEATED:
          logging.warning('Default value for repeated field %s is not being '
                          'written to proto file' % field.name)
        else:
          # Convert default value to string.
          if field.variant == messages.Variant.MESSAGE:
            logging.warning(
              'Message field %s should not have default values' % field.name)
            default = None
          elif field.variant == messages.Variant.STRING:
            default = repr(field.default_value.encode('utf-8'))
          elif field.variant == messages.Variant.BYTES:
            default = repr(field.default_value)
          else:
            default = str(field.default_value)

          if default is not None:
            default_format = ' [default=%s]' % default

      if field.variant in (messages.Variant.MESSAGE, messages.Variant.ENUM):
        field_type = field.type_name
      else:
        field_type = str(field.variant).lower()
            
      out << '%s %s %s = %s%s;' % (str(field.label).lower(),
                                   field_type,
                                   field.name,
                                   field.number,
                                   default_format)

  def write_messages(message_descriptors):
    """Write nested and non-nested Message types.

    Args:
      message_descriptors: List of MessageDescriptor objects from which to
        generate messages.
    """
    for message in message_descriptors or []:
      out << ''
      out << ''
      out << 'message %s {' % message.name

      with out.indent():
        if message.enum_types:
          write_enums(message.enum_types)

        if message.message_types:
          write_messages(message.message_types)

        if message.fields:
          write_fields(message.fields)

      out << '}'

  write_messages(file_descriptor.message_types)



#!/usr/bin/env python
#
# Copyright 2010 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""URL encoding support for messages types.

Protocol support for URL encoded form parameters.

Nested Fields:
  Nested fields are repesented by dot separated names.  For example, consider
  the following messages:

    class WebPage(Message):

      title = StringField(1)
      tags = StringField(2, repeated=True)

    class WebSite(Message):

      name = StringField(1)
      home = MessageField(WebPage, 2)
      pages = MessageField(WebPage, 3, repeated=True)

  And consider the object:

    page = WebPage()
    page.title = 'Welcome to NewSite 2010'

    site = WebSite()
    site.name = 'NewSite 2010'
    site.home = page

  The URL encoded representation of this constellation of objects is.

    name=NewSite+2010&home.title=Welcome+to+NewSite+2010

  An object that exists but does not have any state can be represented with
  a reference to its name alone with no value assigned to it.  For example:

    page = WebSite()
    page.name = 'My Empty Site'
    page.home = WebPage()

  is represented as:

    name=My+Empty+Site&home=

  This represents a site with an empty uninitialized home page.

Repeated Fields:
  Repeated fields are represented by the name of and the index of each value
  separated by a dash.  For example, consider the following message:

    home = Page()
    home.title = 'Nome'

    news = Page()
    news.title = 'News'
    news.tags = ['news', 'articles']

    instance = WebSite()
    instance.name = 'Super fun site'
    instance.pages = [home, news, preferences]

  An instance of this message can be represented as:

    name=Super+fun+site&page-0.title=Home&pages-1.title=News&...
    pages-1.tags-0=new&pages-1.tags-1=articles

Helper classes:

  URLEncodedRequestBuilder: Used for encapsulating the logic used for building
    a request message from a URL encoded RPC.
"""
import six

__author__ = 'rafek@google.com (Rafe Kaplan)'

import cgi
import re
import urllib

from . import message_types
from . import messages
from . import util

__all__ = ['CONTENT_TYPE',
           'URLEncodedRequestBuilder',
           'encode_message',
           'decode_message',
           ]

CONTENT_TYPE = 'application/x-www-form-urlencoded'

_FIELD_NAME_REGEX = re.compile(r'^([a-zA-Z_][a-zA-Z_0-9]*)(?:-([0-9]+))?$')


class URLEncodedRequestBuilder(object):
  """Helper that encapsulates the logic used for building URL encoded messages.

  This helper is used to map query parameters from a URL encoded RPC to a
  message instance.
  """

  @util.positional(2)
  def __init__(self, message, prefix=''):
    """Constructor.

    Args:
      message: Message instance to build from parameters.
      prefix: Prefix expected at the start of valid parameters.
    """
    self.__parameter_prefix = prefix

    # The empty tuple indicates the root message, which has no path.
    # __messages is a full cache that makes it very easy to look up message
    # instances by their paths.  See make_path for details about what a path
    # is.
    self.__messages = {(): message}

    # This is a cache that stores paths which have been checked for
    # correctness.  Correctness means that an index is present for repeated
    # fields on the path and absent for non-repeated fields.  The cache is
    # also used to check that indexes are added in the right order so that
    # dicontiguous ranges of indexes are ignored.
    self.__checked_indexes = set([()])

  def make_path(self, parameter_name):
    """Parse a parameter name and build a full path to a message value.

    The path of a method is a tuple of 2-tuples describing the names and
    indexes within repeated fields from the root message (the message being
    constructed by the builder) to an arbitrarily nested message within it.

    Each 2-tuple node of a path (name, index) is:
      name: The name of the field that refers to the message instance.
      index: The index within a repeated field that refers to the message
        instance, None if not a repeated field.

    For example, consider:

      class VeryInner(messages.Message):
        ...

      class Inner(messages.Message):

        very_inner = messages.MessageField(VeryInner, 1, repeated=True)

      class Outer(messages.Message):

        inner = messages.MessageField(Inner, 1)

    If this builder is building an instance of Outer, that instance is
    referred to in the URL encoded parameters without a path.  Therefore
    its path is ().

    The child 'inner' is referred to by its path (('inner', None)).

    The first child of repeated field 'very_inner' on the Inner instance
    is referred to by (('inner', None), ('very_inner', 0)).

    Examples:
      # Correct reference to model where nation is a Message, district is
      # repeated Message and county is any not repeated field type.
      >>> make_path('nation.district-2.county')
      (('nation', None), ('district', 2), ('county', None))

      # Field is not part of model.
      >>> make_path('nation.made_up_field')
      None

      # nation field is not repeated and index provided.
      >>> make_path('nation-1')
      None

      # district field is repeated and no index provided.
      >>> make_path('nation.district')
      None

    Args:
      parameter_name: Name of query parameter as passed in from the request.
        in order to make a path, this parameter_name must point to a valid
        field within the message structure.  Nodes of the path that refer to
        repeated fields must be indexed with a number, non repeated nodes must
        not have an index.

    Returns:
      Parsed version of the parameter_name as a tuple of tuples:
        attribute: Name of attribute associated with path.
        index: Postitive integer index when it is a repeated field, else None.
      Will return None if the parameter_name does not have the right prefix,
      does not point to a field within the message structure, does not have
      an index if it is a repeated field or has an index but is not a repeated
      field.
    """
    if parameter_name.startswith(self.__parameter_prefix):
      parameter_name = parameter_name[len(self.__parameter_prefix):]
    else:
      return None

    path = []
    name = []
    message_type = type(self.__messages[()])  # Get root message.

    for item in parameter_name.split('.'):
      # This will catch sub_message.real_message_field.not_real_field
      if not message_type:
        return None

      item_match = _FIELD_NAME_REGEX.match(item)
      if not item_match:
        return None
      attribute = item_match.group(1)
      index = item_match.group(2)
      if index:
        index = int(index)

      try:
        field = message_type.field_by_name(attribute)
      except KeyError:
        return None

      if field.repeated != (index is not None):
        return None

      if isinstance(field, messages.MessageField):
        message_type = field.message_type
      else:
        message_type = None

      # Path is valid so far.  Append node and continue.
      path.append((attribute, index))

    return tuple(path)

  def __check_index(self, parent_path, name, index):
    """Check correct index use and value relative to a given path.

    Check that for a given path the index is present for repeated fields
    and that it is in range for the existing list that it will be inserted
    in to or appended to.

    Args:
      parent_path: Path to check against name and index.
      name: Name of field to check for existance.
      index: Index to check.  If field is repeated, should be a number within
        range of the length of the field, or point to the next item for
        appending.
    """
    # Don't worry about non-repeated fields.
    # It's also ok if index is 0 because that means next insert will append.
    if not index:
      return True

    parent = self.__messages.get(parent_path, None)
    value_list = getattr(parent, name, None)
    # If the list does not exist then the index should be 0.  Since it is
    # not, path is not valid.
    if not value_list:
      return False

    # The index must either point to an element of the list or to the tail.
    return len(value_list) >= index

  def __check_indexes(self, path):
    """Check that all indexes are valid and in the right order.

    This method must iterate over the path and check that all references
    to indexes point to an existing message or to the end of the list, meaning
    the next value should be appended to the repeated field.

    Args:
      path: Path to check indexes for.  Tuple of 2-tuples (name, index).  See
        make_path for more information.

    Returns:
      True if all the indexes of the path are within range, else False.
    """
    if path in self.__checked_indexes:
      return True

    # Start with the root message.
    parent_path = ()

    for name, index in path:
      next_path = parent_path + ((name, index),)
      # First look in the checked indexes cache.
      if next_path not in self.__checked_indexes:
        if not self.__check_index(parent_path, name, index):
          return False
        self.__checked_indexes.add(next_path)

      parent_path = next_path

    return True

  def __get_or_create_path(self, path):
    """Get a message from the messages cache or create it and add it.

    This method will also create any parent messages based on the path.

    When a new instance of a given message is created, it is stored in
    __message by its path.

    Args:
      path: Path of message to get.  Path must be valid, in other words
        __check_index(path) returns true.  Tuple of 2-tuples (name, index).
        See make_path for more information.

    Returns:
      Message instance if the field being pointed to by the path is a
      message, else will return None for non-message fields.
    """
    message = self.__messages.get(path, None)
    if message:
      return message

    parent_path = ()
    parent = self.__messages[()]  # Get the root object

    for name, index in path:
      field = parent.field_by_name(name)
      next_path = parent_path + ((name, index),)
      next_message = self.__messages.get(next_path, None)
      if next_message is None:
        next_message = field.message_type()
        self.__messages[next_path] = next_message
        if not field.repeated:
          setattr(parent, field.name, next_message)
        else:
          list_value = getattr(parent, field.name, None)
          if list_value is None:
            setattr(parent, field.name, [next_message])
          else:
            list_value.append(next_message)

      parent_path = next_path
      parent = next_message

    return parent

  def add_parameter(self, parameter, values):
    """Add a single parameter.

    Adds a single parameter and its value to the request message.

    Args:
      parameter: Query string parameter to map to request.
      values: List of values to assign to request message.

    Returns:
      True if parameter was valid and added to the message, else False.

    Raises:
      DecodeError if the parameter refers to a valid field, and the values
      parameter does not have one and only one value.  Non-valid query
      parameters may have multiple values and should not cause an error.
    """
    path = self.make_path(parameter)

    if not path:
      return False

    # Must check that all indexes of all items in the path are correct before
    # instantiating any of them.  For example, consider:
    #
    #   class Repeated(object):
    #     ...
    #
    #   class Inner(object):
    #
    #     repeated = messages.MessageField(Repeated, 1, repeated=True)
    #
    #   class Outer(object):
    #
    #     inner = messages.MessageField(Inner, 1)
    #
    #   instance = Outer()
    #   builder = URLEncodedRequestBuilder(instance)
    #   builder.add_parameter('inner.repeated')
    #
    #   assert not hasattr(instance, 'inner')
    #
    # The check is done relative to the instance of Outer pass in to the
    # constructor of the builder.  This instance is not referred to at all
    # because all names are assumed to be relative to it.
    #
    # The 'repeated' part of the path is not correct because it is missing an
    # index.  Because it is missing an index, it should not create an instance
    # of Repeated.  In this case add_parameter will return False and have no
    # side effects.
    #
    # A correct path that would cause a new Inner instance to be inserted at
    # instance.inner and a new Repeated instance to be appended to the
    # instance.inner.repeated list would be 'inner.repeated-0'.
    if not self.__check_indexes(path):
      return False

    # Ok to build objects.
    parent_path = path[:-1]
    parent = self.__get_or_create_path(parent_path)
    name, index = path[-1]
    field = parent.field_by_name(name)

    if len(values) != 1:
      raise messages.DecodeError(
          'Found repeated values for field %s.' % field.name)

    value = values[0]

    if isinstance(field, messages.IntegerField):
      converted_value = int(value)
    elif isinstance(field, message_types.DateTimeField):
      try:
        converted_value = util.decode_datetime(value)
      except ValueError as e:
        raise messages.DecodeError(e)
    elif isinstance(field, messages.MessageField):
      # Just make sure it's instantiated.  Assignment to field or
      # appending to list is done in __get_or_create_path.
      self.__get_or_create_path(path)
      return True
    elif isinstance(field, messages.StringField):
      converted_value = value.decode('utf-8')
    elif isinstance(field, messages.BooleanField):
      converted_value = value.lower() == 'true' and True or False
    else:
      try:
        converted_value = field.type(value)
      except TypeError:
        raise messages.DecodeError('Invalid enum value "%s"' % value)

    if field.repeated:
      value_list = getattr(parent, field.name, None)
      if value_list is None:
        setattr(parent, field.name, [converted_value])
      else:
        if index == len(value_list):
          value_list.append(converted_value)
        else:
          # Index should never be above len(value_list) because it was
          # verified during the index check above.
          value_list[index] = converted_value
    else:
      setattr(parent, field.name, converted_value)

    return True


@util.positional(1)
def encode_message(message, prefix=''):
  """Encode Message instance to url-encoded string.

  Args:
    message: Message instance to encode in to url-encoded string.
    prefix: Prefix to append to field names of contained values.

  Returns:
    String encoding of Message in URL encoded format.

  Raises:
    messages.ValidationError if message is not initialized.
  """
  message.check_initialized()

  parameters = []
  def build_message(parent, prefix):
    """Recursively build parameter list for URL response.

    Args:
      parent: Message to build parameters for.
      prefix: Prefix to append to field names of contained values.

    Returns:
      True if some value of parent was added to the parameters list,
      else False, meaning the object contained no values.
    """
    has_any_values = False
    for field in sorted(parent.all_fields(), key=lambda f: f.number):
      next_value = parent.get_assigned_value(field.name)
      if next_value is None:
        continue

      # Found a value.  Ultimate return value should be True.
      has_any_values = True

      # Normalize all values in to a list.
      if not field.repeated:
        next_value = [next_value]

      for index, item in enumerate(next_value):
        # Create a name with an index if it is a repeated field.
        if field.repeated:
          field_name = '%s%s-%s' % (prefix, field.name, index)
        else:
          field_name = prefix + field.name

        if isinstance(field, message_types.DateTimeField):
          # DateTimeField stores its data as a RFC 3339 compliant string.
          parameters.append((field_name, item.isoformat()))
        elif isinstance(field, messages.MessageField):
          # Message fields must be recursed in to in order to construct
          # their component parameter values.
          if not build_message(item, field_name + '.'):
            # The nested message is empty.  Append an empty value to
            # represent it.
            parameters.append((field_name, ''))
        elif isinstance(field, messages.BooleanField):
          parameters.append((field_name, item and 'true' or 'false'))
        else:
          if isinstance(item, six.text_type):
            item = item.encode('utf-8')
          parameters.append((field_name, str(item)))

    return has_any_values

  build_message(message, prefix)

  # Also add any unrecognized values from the decoded string.
  for key in message.all_unrecognized_fields():
    values, _ = message.get_unrecognized_field_info(key)
    if not isinstance(values, (list, tuple)):
      values = (values,)
    for value in values:
      parameters.append((key, value))

  return urllib.urlencode(parameters)


def decode_message(message_type, encoded_message, **kwargs):
  """Decode urlencoded content to message.

  Args:
    message_type: Message instance to merge URL encoded content into.
    encoded_message: URL encoded message.
    prefix: Prefix to append to field names of contained values.

  Returns:
    Decoded instance of message_type.
  """
  message = message_type()
  builder = URLEncodedRequestBuilder(message, **kwargs)
  arguments = cgi.parse_qs(encoded_message, keep_blank_values=True)
  for argument, values in sorted(six.iteritems(arguments)):
    added = builder.add_parameter(argument, values)
    # Save off any unknown values, so they're still accessible.
    if not added:
      message.set_unrecognized_field(argument, values, messages.Variant.STRING)
  message.check_initialized()
  return message
